package com.qmth.distributed.print.business.bean.params;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.teachcloud.common.annotation.EditKey;
import io.swagger.annotations.ApiModelProperty;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.Range;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

/**
 * @Description: 考生字典参数
 * @Author: CaoZixuan
 * @Date: 2024-02-22
 */
public class BasicExamStudentParam {

    @ApiModelProperty("id(编辑必填)")
    @JsonSerialize(using = ToStringSerializer.class)
    @EditKey
    private Long id;

    @ApiModelProperty("学期id（必填）")
    @JsonSerialize(using = ToStringSerializer.class)
    @NotNull(message = "请选择学期")
    private Long semesterId;

    @ApiModelProperty("考试id（必填）")
    @JsonSerialize(using = ToStringSerializer.class)
    @NotNull(message = "请选择考试")
    private Long examId;

    @ApiModelProperty("课程代码（必填）")
    @NotNull(message = "请选择课程")
    private Long courseId;

    @ApiModelProperty("姓名（必填）")
    @NotNull(message = "请输入姓名")
    @Length(min = 1, message = "请输入姓名")
    private String studentName;

    @ApiModelProperty("学号（必填）")
    @NotNull(message = "请输入学号")
    @Length(min = 1, message = "请输入学号")
    private String studentCode;

    @ApiModelProperty("座位号")
    private String siteNumber;

    @ApiModelProperty("学院")
    @NotNull(message = "请输入学院")
    @Length(min = 1, message = "请输入学院")
    private String collegeName;

    @ApiModelProperty("专业")
    private String majorName;

    @ApiModelProperty("教学班")
    @NotNull(message = "请输入教学班")
    @Length(min = 1, message = "请输入教学班")
    private String teachClassName;
    @ApiModelProperty("行政班")
    private String className;

    @ApiModelProperty("试卷编号")
    private String paperNumber;

    @ApiModelProperty("任课老师")
    private String teacherName;

    @ApiModelProperty("任课老师工号")
    private String teacherCode;

    @ApiModelProperty("考试开始时间")
    @NotNull(message = "请选择考试时间")
    private Long examStartTime;

    @ApiModelProperty("考试结束时间")
    @NotNull(message = "请选择考试时间")
    private Long examEndTime;

    @ApiModelProperty("校区（考点）")
    private String examPlace;

    @ApiModelProperty("考试教室（考场）")
    @NotBlank(message = "请输入考试教室（考场）")
    private String examRoom;
    @ApiModelProperty("扩展字段")
    private String extendFields;

    @ApiModelProperty("卷型")
    private String paperType;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getSemesterId() {
        return semesterId;
    }

    public void setSemesterId(Long semesterId) {
        this.semesterId = semesterId;
    }

    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public Long getCourseId() {
        return courseId;
    }

    public void setCourseId(Long courseId) {
        this.courseId = courseId;
    }

    public String getStudentName() {
        return studentName;
    }

    public void setStudentName(String studentName) {
        this.studentName = studentName;
    }

    public String getStudentCode() {
        return studentCode;
    }

    public void setStudentCode(String studentCode) {
        this.studentCode = studentCode;
    }

    public String getSiteNumber() {
        return siteNumber;
    }

    public void setSiteNumber(String siteNumber) {
        this.siteNumber = siteNumber;
    }

    public String getCollegeName() {
        return collegeName;
    }

    public void setCollegeName(String collegeName) {
        this.collegeName = collegeName;
    }

    public String getMajorName() {
        return majorName;
    }

    public void setMajorName(String majorName) {
        this.majorName = majorName;
    }

    public String getTeachClassName() {
        return teachClassName;
    }

    public void setTeachClassName(String teachClassName) {
        this.teachClassName = teachClassName;
    }

    public String getClassName() {
        return className;
    }

    public void setClassName(String className) {
        this.className = className;
    }

    public String getPaperNumber() {
        return paperNumber;
    }

    public void setPaperNumber(String paperNumber) {
        this.paperNumber = paperNumber;
    }

    public String getTeacherName() {
        return teacherName;
    }

    public void setTeacherName(String teacherName) {
        this.teacherName = teacherName;
    }

    public String getTeacherCode() {
        return teacherCode;
    }

    public void setTeacherCode(String teacherCode) {
        this.teacherCode = teacherCode;
    }

    public Long getExamStartTime() {
        return examStartTime;
    }

    public void setExamStartTime(Long examStartTime) {
        this.examStartTime = examStartTime;
    }

    public Long getExamEndTime() {
        return examEndTime;
    }

    public void setExamEndTime(Long examEndTime) {
        this.examEndTime = examEndTime;
    }

    public String getExamPlace() {
        return examPlace;
    }

    public void setExamPlace(String examPlace) {
        this.examPlace = examPlace;
    }

    public String getExamRoom() {
        return examRoom;
    }

    public void setExamRoom(String examRoom) {
        this.examRoom = examRoom;
    }

    public String getExtendFields() {
        return extendFields;
    }

    public void setExtendFields(String extendFields) {
        this.extendFields = extendFields;
    }

    public String getPaperType() {
        return paperType;
    }

    public void setPaperType(String paperType) {
        this.paperType = paperType;
    }
}
