package com.qmth.distributed.print.business.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.distributed.print.business.bean.dto.*;
import com.qmth.distributed.print.business.bean.dto.approvalForm.ExamPackageDetail;
import com.qmth.distributed.print.business.bean.result.WorkResult;
import com.qmth.distributed.print.business.bean.result.statistics.PrintCountResult;
import com.qmth.distributed.print.business.bean.result.statistics.ScanCalculateResult;
import com.qmth.distributed.print.business.bean.result.statistics.ScanCountResult;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.enums.PrintPlanStatusEnum;
import com.qmth.teachcloud.common.bean.dto.BlurryUserDto;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * <p>
 * 命题任务 Mapper 接口
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
public interface ExamTaskMapper extends BaseMapper<ExamTask> {

    IPage<ExamTaskDto> listPage(Page<ExamTaskDto> page, @Param("schoolId") Long schoolId, @Param("semesterId") Long semesterId, @Param("examId") Long examId, @Param("enable") Boolean enable, @Param("status") String status, @Param("courseId") Long courseId, @Param("paperNumber") String paperNumber, @Param("startTime") Long startTime, @Param("endTime") Long endTime, @Param("userName") String userName, @Param("dpr") DataPermissionRule dpr);

    List<BlurryUserDto> listUser(@Param("schoolId") Long schoolId, @Param("param") String param, @Param("orgIds") Set<Long> orgIds);

    IPage<ExamTaskDto> listTaskApply(Page<ExamTaskDto> page, @Param("schoolId") Long schoolId, @Param("semesterId") Long semesterId, @Param("examId") Long examId, @Param("auditStatus") String auditStatus, @Param("courseId") Long courseId, @Param("paperNumber") String paperNumber, @Param("startTime") Long startTime, @Param("endTime") Long endTime, @Param("userId") Long userId, @Param("userName") String userName);

    IPage<ExamTaskDto> listTaskReviewUnaudited(Page<ExamTaskDto> page, @Param("schoolId") Long schoolId, @Param("semesterId") Long semesterId, @Param("examId") Long examId, @Param("courseId") Long courseId, @Param("paperNumber") String paperNumber, @Param("userId") Long userId, @Param("startTime") Long startTime, @Param("endTime") Long endTime, @Param("startCreateTime") Long startCreateTime, @Param("endCreateTime") Long endCreateTime, @Param("currentUserId") Long currentUserId);

    IPage<ExamTaskDto> listTaskReviewAudited(Page<ExamTaskDto> page, @Param("schoolId") Long schoolId, @Param("semesterId") Long semesterId, @Param("examId") Long examId, @Param("courseId") Long courseId, @Param("paperNumber") String paperNumber, @Param("userId") Long userId, @Param("startTime") Long startTime, @Param("endTime") Long endTime, @Param("orgIds") Set<Long> orgIds, @Param("startCreateTime") Long startCreateTime, @Param("endCreateTime") Long endCreateTime, @Param("currentUserId") Long currentUserId);

    IPage<ExamTaskDetailDto> listTaskPaper(Page<ExamTaskDetailDto> page, @Param("schoolId") Long schoolId, @Param("semesterId") Long semesterId, @Param("examId") Long examId, @Param("courseId") Long courseId, @Param("paperNumber") String paperNumber, @Param("startTime") Long startTime, @Param("endTime") Long endTime, @Param("dpr") DataPermissionRule dpr, @Param("containsQuestionTeacher") boolean containsQuestionTeacher, @Param("userId") Long userId);

    ExamTaskDetailCardDto applyGetOne(@Param("examTaskId") Long examTaskId, @Param("source") String source);

    List<ExamTask> listExamTaskExpire(@Param("startTime") Long startTime, @Param("endTime") Long endTime, @Param("status") String[] status);

    List<ExamTask> listExamTaskAuditExpire(@Param("startTime") Long startTime, @Param("endTime") Long endTime);

    List<ExamTaskDetailDto> listTaskPaper(@Param("schoolId") Long schoolId, @Param("semesterId") Long semesterId, @Param("examId") Long examId, @Param("courseId") Long courseId, @Param("paperNumber") String paperNumber, @Param("startTime") Long startTime, @Param("endTime") Long endTime, @Param("dpr") DataPermissionRule dpr, @Param("orgIds") Set<Long> orgIds, @Param("containsQuestionTeacher") boolean containsQuestionTeacher, @Param("userId") Long userId);

    /**
     * 获取待办任务
     *
     * @return
     */
    public IPage<WorkResult> getFlowTaskReadyList(IPage<Map> iPage, @Param("schoolId") Long schoolId, @Param("orgId") Long orgId, @Param("userId") String userId);

    /**
     * 根据流程状态查找命题任务
     */
    List<ExamTask> findExamTaskByFlowStatus(@Param("schoolId") Long schoolId, @Param("examId") Long examId, @Param("courseId") Long courseId, @Param("paperNumber") String paperNumber, @Param("flowStatus") String flowStatus);

    /**
     * 根据试卷编号查询考场详细数据集（印刷计划状态为‘PRINTING’、‘PRINTING_FINISH‘的）
     *
     * @param paperNumber             试卷编号
     * @param printPlanStatusEnumList 印刷计划状态集合
     * @return 数据集
     */
    List<ExamPackageDetail> findExamPackageDetailByPaperNumber(@Param("schoolId") Long schoolId, @Param("examId") Long examId, @Param("courseId") Long courseId, @Param("paperNumber") String paperNumber, @Param("printPlanStatusEnumList") List<PrintPlanStatusEnum> printPlanStatusEnumList);

    ExamTask findByFlowId(@Param("flowId") Long flowId);

    ExamTaskDto getByExamTaskId(Long examTaskId);

    IPage<ExamTaskDetailDto> listExamTaskDetailDownload(@Param("page") Page<ExamTaskDetailDto> page, @Param("examId") Long examId, @Param("courseId") Long courseId, @Param("paperNumber") String paperNumber, @Param("orgId") Long orgId, @Param("userParam") String userParam, @Param("dpr") DataPermissionRule dpr);

    /**
     * 根据条件查询要下载的试卷文件
     *
     * @param semesterId 学期id
     * @param examId     考试id
     * @param courseName 课程明后才能
     * @param idSet      命题任务id集合
     * @param dpr
     * @return 命题任务试卷信息
     */
    List<ExamTaskDetailDto> listExamTaskDetailDownloadByQuery(@Param("semesterId") Long semesterId, @Param("examId") Long examId, @Param("courseName") String courseName, @Param("orgId") Long orgId, @Param("idSet") Set<Long> idSet, @Param("dpr") DataPermissionRule dpr);

    /**
     * 根据流程id和任务节点id获取待办
     *
     * @param flowId
     * @param taskId
     * @return
     */
    public WorkResult getFlowInfo(@Param("flowId") Long flowId, @Param("taskId") Long taskId);

    IPage<MakeupExamTaskDto> listMakeupExamTask(@Param("page") Page<MakeupExamTaskDto> page, @Param("printPlanId") Long printPlanId, @Param("semesterId") Long semesterId, @Param("examId") Long examId, @Param("collegeId") Long collegeId, @Param("courseId") Long courseId, @Param("paperNumber") String paperNumber, @Param("userName") String userName);

    /**
     * 命题任务统计查询
     *
     * @param iPage
     * @param schoolId
     * @param semesterId
     * @param examId
     * @param printPlanId
     * @param collegeId
     * @param courseId
     * @param paperNumber
     * @param cardType
     * @param finishStatus
     * @param dpr
     * @return
     */
    IPage<PrintCountResult> printCountList(IPage<Map> iPage, @Param("schoolId") Long schoolId, @Param("semesterId") Long semesterId, @Param("examId") Long examId, @Param("printPlanId") Long printPlanId, @Param("collegeId") Long collegeId, @Param("courseId") Long courseId, @Param("paperNumber") String paperNumber, @Param("cardType") String cardType, @Param("finishStatus") String finishStatus, @Param("dpr") DataPermissionRule dpr);

    /**
     * 命题任务统计导出
     *
     * @param schoolId
     * @param semesterId
     * @param examId
     * @param printPlanId
     * @param collegeId
     * @param courseId
     * @param paperNumber
     * @param cardType
     * @param finishStatus
     * @param dpr
     * @return
     */
    List<PrintCountResult> printCountListExport(@Param("schoolId") Long schoolId, @Param("semesterId") Long semesterId, @Param("examId") Long examId, @Param("printPlanId") Long printPlanId, @Param("collegeId") Long collegeId, @Param("courseId") Long courseId, @Param("paperNumber") String paperNumber, @Param("cardType") String cardType, @Param("finishStatus") String finishStatus, @Param("dpr") DataPermissionRule dpr);

    /**
     * 印刷任务统计查询
     *
     * @param iPage
     * @param schoolId
     * @param semesterId
     * @param examId
     * @param collegeId
     * @param courseId
     * @param paperNumber
     * @param cardType
     * @param dpr
     * @return
     */
    IPage<ScanCountResult> scanCountList(IPage<Map> iPage, @Param("schoolId") Long schoolId, @Param("semesterId") Long semesterId, @Param("examId") Long examId, @Param("collegeId") Long collegeId, @Param("courseId") Long courseId, @Param("paperNumber") String paperNumber, @Param("cardType") String cardType, @Param("dpr") DataPermissionRule dpr);

    /**
     * 印刷任务统计导出
     *
     * @param schoolId
     * @param semesterId
     * @param examId
     * @param collegeId
     * @param courseId
     * @param paperNumber
     * @param cardType
     * @param dpr
     * @return
     */
    List<ScanCountResult> scanCountListExport(@Param("schoolId") Long schoolId, @Param("semesterId") Long semesterId, @Param("examId") Long examId, @Param("collegeId") Long collegeId, @Param("courseId") Long courseId, @Param("paperNumber") String paperNumber, @Param("cardType") String cardType, @Param("dpr") DataPermissionRule dpr);

    /**
     * 统计计算
     *
     * @param examId
     * @param courseId
     * @param paperNumber
     * @param studentCount
     * @return
     */
    public ScanCalculateResult scanCalculate(@Param("examId") Long examId, @Param("courseId") Long courseId, @Param("paperNumber") String paperNumber, @Param("studentCount") int studentCount);
}
