package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.entity.ExamDetailCourse;
import com.qmth.distributed.print.business.entity.ExamStudent;
import com.qmth.distributed.print.business.enums.ExamDetailStatusEnum;
import com.qmth.distributed.print.business.enums.ExamStatusEnum;
import com.qmth.distributed.print.business.mapper.ExamDetailCourseMapper;
import com.qmth.distributed.print.business.service.ExamDetailCourseService;
import com.qmth.teachcloud.common.bean.result.TbTaskDetailResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.service.BasicCourseService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * <p>
 * 考务数据（考场关联科目） 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamDetailCourseServiceImpl extends ServiceImpl<ExamDetailCourseMapper, ExamDetailCourse> implements ExamDetailCourseService {

    @Resource
    private BasicCourseService basicCourseService;


    @Override
    public List<Map<String, Object>> listClientByExamDetailIdAndStatus(Long examDetailId) {
        return this.baseMapper.listClientByExamDetailIdAndStatus(examDetailId, ExamDetailStatusEnum.FINISH.name());
    }

    @Override
    public void updatePaperNumber(List<Long> examDetailCourseIds, String paperNumber, String relatePaperType) {
        UpdateWrapper<ExamDetailCourse> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamDetailCourse::getPaperNumber, paperNumber)
                .set(ExamDetailCourse::getPaperType, relatePaperType)
                .in(ExamDetailCourse::getId, examDetailCourseIds);
        this.update(updateWrapper);
    }

    @Override
    public List<ExamDetailCourse> listByExamDetailIdAndStatus(Long examDetailId) {
        return this.baseMapper.listByExamDetailIdAndStatus(examDetailId, ExamStatusEnum.SUBMIT.name());
    }

    @Override
    public List<BasicCourse> listCoursesByPrintPlanId(String param, List<Long> printPlanIdList) {
        return this.baseMapper.listCoursesByPrintPlanId(SystemConstant.translateSpecificSign(param), printPlanIdList);
    }

    @Override
    public List<ExamDetailCourse> listByPrintPlanIdAndCourseIdAndPaperNumber(Long schoolId, Long printPlanId, Long courseId, String paperNumber) {
        return this.baseMapper.listByPrintPlanIdAndCourseCodeAndPaperNumber(schoolId, printPlanId, courseId, paperNumber);
    }

    @Override
    public List<ExamDetailCourse> listExamDetailByExamIdAndPaperNumber(Long schoolId, Long examId, String paperNumber) {
        return this.baseMapper.listExamDetailByExamIdAndPaperNumber(schoolId, examId, paperNumber);
    }

    @Override
    public TbTaskDetailResult getByExamDetailId(Long examDetailId) {
        return this.baseMapper.getByExamDetailId(examDetailId);
    }

    @Override
    public List<String> listPaperNumberByPrintPlanId(String param, List<Long> printPlanIdList, Set<Long> orgIds) {
        return this.baseMapper.listPaperNumberByPrintPlanId(SystemConstant.translateSpecificSign(param), printPlanIdList, orgIds);
    }

    @Override
    public List<ExamDetailCourse> listByExamDetailId(Long examDetailId) {
        QueryWrapper<ExamDetailCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamDetailCourse::getExamDetailId, examDetailId);
        List<ExamDetailCourse> examDetailCourseList = this.list(queryWrapper);
        for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
            BasicCourse basicCourse = basicCourseService.getById(examDetailCourse.getCourseId());
            if (basicCourse != null) {
                examDetailCourse.setCourseName(basicCourse.getName());
                examDetailCourse.setCourseCode(basicCourse.getCode());
            }
        }
        return examDetailCourseList;
    }

    @Override
    public List<JSONObject> listExamDetailPaperNumber(Long examDetailId) {
        QueryWrapper<ExamDetailCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamDetailCourse::getExamDetailId, examDetailId)
                .orderByAsc(ExamDetailCourse::getCourseCode);
        List<ExamDetailCourse> examDetailCourseList = this.list(queryWrapper);
        List<JSONObject> jsonObjectList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(examDetailCourseList)) {
            for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("examDetailCourseId", String.valueOf(examDetailCourse.getId()));
                jsonObject.put("courseCode", examDetailCourse.getCourseCode());
                jsonObject.put("courseName", examDetailCourse.getCourseName());
                List<String> paperTypes = new ArrayList<>();
                if (StringUtils.isNotBlank(examDetailCourse.getPaperType())) {
                    paperTypes = Arrays.asList(examDetailCourse.getPaperType().split(","));

                }
                jsonObject.put("paperTypes", paperTypes);
                jsonObjectList.add(jsonObject);
            }
        }
        return jsonObjectList;
    }

    @Override
    public void updateTotalSubjects(Long examDetailCourseId, int totalSubjects) {
        UpdateWrapper<ExamDetailCourse> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamDetailCourse::getTotalSubjects, totalSubjects)
                .eq(ExamDetailCourse::getId, examDetailCourseId);
        this.update(updateWrapper);
    }
}
