package com.qmth.distributed.print.api;

import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.params.ExamTaskReApplyParam;
import com.qmth.distributed.print.business.service.ActivitiService;
import com.qmth.distributed.print.business.service.ExamTaskService;
import com.qmth.distributed.print.business.service.GradeInitializeService;
import com.qmth.teachcloud.common.annotation.OperationLogDetail;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.GradeAnalyzePaperStatusEnum;
import com.qmth.teachcloud.common.enums.log.OperationTypeEnum;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.util.Map;

/**
 * <p>
 * 入库审核 前端控制器
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Api(tags = "入库审核Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_EXAM + "/task")
@Validated
public class ExamTaskAuditController {

    @Resource
    private ExamTaskService examTaskService;
    @Resource
    private ActivitiService activitiService;
    @Resource
    private GradeInitializeService gradeInitializeService;
    @Resource
    private MarkPaperService markPaperService;


    @ApiOperation(value = "审核-印刷员驳回后命题老师修改")
    @RequestMapping(value = "/apply_resave", method = RequestMethod.POST)
    @OperationLogDetail(operationType = OperationTypeEnum.UPDATE)
    public Result taskReApplySave(@RequestBody ExamTaskReApplyParam examTaskReApplyParam) {
        Map<String, Object> map = examTaskService.applyResave(examTaskReApplyParam);
        activitiService.sendFlowTaskApproveMsg(map);
        return ResultUtil.ok();
    }

    @ApiOperation(value = "入库审核-查询-未审核")
    @RequestMapping(value = "/review_list_unaudited", method = RequestMethod.POST)
    public Result taskReviewListUnaudited(@ApiParam(value = "学期ID") @RequestParam(value = "semesterId", required = false) Long semesterId,
                                          @ApiParam(value = "考试ID") @RequestParam(value = "examId", required = false) Long examId,
                                          @ApiParam(value = "课程ID") @RequestParam(value = "courseId", required = false) Long courseId,
                                          @ApiParam(value = "试卷编号") @RequestParam(value = "paperNumber", required = false) String paperNumber,
                                          @ApiParam(value = "命题老师ID") @RequestParam(value = "userId", required = false) Long userId,
                                          @ApiParam(value = "命题时间（开始）") @RequestParam(value = "startTime", required = false) Long startTime,
                                          @ApiParam(value = "命题时间（结束）") @RequestParam(value = "endTime", required = false) Long endTime,
                                          @ApiParam(value = "申请时间（开始）") @RequestParam(value = "startCreateTime", required = false) Long startCreateTime,
                                          @ApiParam(value = "申请时间（结束）") @RequestParam(value = "endCreateTime", required = false) Long endCreateTime,
                                          @ApiParam(value = "分页参数") @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                          @ApiParam(value = "分页参数") @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        if (semesterId == null) {
            throw ExceptionResultEnum.SEMESTER_ID_IS_NULL.exception();
        }
        if (examId == null) {
            throw ExceptionResultEnum.EXAM_ID_IS_NULL.exception();
        }
        return ResultUtil.ok(examTaskService.listTaskReviewUnaudited(semesterId, examId, courseId, paperNumber, userId, startTime, endTime, startCreateTime, endCreateTime, pageNumber, pageSize));
    }

    @ApiOperation(value = "入库审核-查询-已审核")
    @RequestMapping(value = "/review_list_audited", method = RequestMethod.POST)
    public Result taskReviewListAudited(@ApiParam(value = "学期ID") @RequestParam(value = "semesterId", required = false) Long semesterId,
                                        @ApiParam(value = "考试ID") @RequestParam(value = "examId", required = false) Long examId,
                                        @ApiParam(value = "课程ID") @RequestParam(value = "courseId", required = false) Long courseId,
                                        @ApiParam(value = "试卷编号") @RequestParam(value = "paperNumber", required = false) String paperNumber,
                                        @ApiParam(value = "命题老师ID") @RequestParam(value = "userId", required = false) Long userId,
                                        @ApiParam(value = "命题时间（开始）") @RequestParam(value = "startTime", required = false) Long startTime,
                                        @ApiParam(value = "命题时间（结束）") @RequestParam(value = "endTime", required = false) Long endTime,
                                        @ApiParam(value = "申请时间（开始）") @RequestParam(value = "startCreateTime", required = false) Long startCreateTime,
                                        @ApiParam(value = "申请时间（结束）") @RequestParam(value = "endCreateTime", required = false) Long endCreateTime,
                                        @ApiParam(value = "分页参数") @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                        @ApiParam(value = "分页参数") @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        if (examId == null) {
            throw ExceptionResultEnum.EXAM_ID_IS_NULL.exception();
        }
        return ResultUtil.ok(examTaskService.listTaskReviewAudited(semesterId, examId, courseId, paperNumber, userId, startTime, endTime, startCreateTime, endCreateTime, pageNumber, pageSize));
    }

    /**
     * 入库审核-查询-已审核数量
     *
     * @return
     */
    @ApiOperation(value = "入库审核-查询-已审核数量")
    @RequestMapping(value = "/review_list_audited_count", method = RequestMethod.POST)
    public Result taskReviewListAuditedCount() {
        int taskReviewAuditedCount = examTaskService.listTaskReviewAudited(null, null, null, null, null, null, null, null, null, SystemConstant.PAGE_NUMBER, SystemConstant.PAGE_SIZE).getRecords().size();
        long paperCount = markPaperService.countByPropositionTeacherId(true);
        long gradeCount = gradeInitializeService.countByPropositionTeacherId(GradeAnalyzePaperStatusEnum.SETTING_GRADE_PAPER_PARAM, true);
        return ResultUtil.ok(taskReviewAuditedCount + paperCount + gradeCount);
    }
}

