package com.qmth.distributed.print.util;

import com.aspose.words.*;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.ImportTemplateEnum;
import com.qmth.teachcloud.common.util.FileUtil;
import com.qmth.teachcloud.common.util.ResultUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Objects;

/**
 * @Description: aspose pdf util
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/4/7
 */
@Component
public class WordToPdfUtil {
    private final static Logger log = LoggerFactory.getLogger(WordToPdfUtil.class);

    /**
     * 获取license
     *
     * @return
     */
    private boolean getLicense() {
        boolean result = false;
        try {
            License aposeLic = new License();
            InputStream inputStream = FileUtil.getStream(ImportTemplateEnum.WORD_TO_PDF_LICENSE.getTemplateName());
            aposeLic.setLicense(inputStream);
            result = true;
        } catch (Exception e) {
            log.error("请求出错", e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return result;
    }

    /**
     * document转pdf
     *
     * @param documentPath
     * @param pdfPath
     * @return
     * @throws IOException
     */
    public File wordToPdf(String documentPath, String pdfPath, int paperSize) throws IOException {
        if (!getLicense()) { // 验证License 若不验证则转化出的pdf文档会有水印产生
            throw ExceptionResultEnum.ERROR.exception("html转pdf失败，验证过期");
        }
        FileOutputStream os = null;
        File file = null;
        try {
            long start = System.currentTimeMillis();
            file = new File(pdfPath); //新建一个pdf文档
            if (!file.getParentFile().exists()) {
                file.getParentFile().mkdirs();
                file.createNewFile();
            }
            os = new FileOutputStream(file);
            Document doc = new Document(documentPath); //Address是将要被转化的word文档
            DocumentBuilder builder = new DocumentBuilder(doc);
            PageSetup pageSetup = builder.getPageSetup();
            pageSetup.setPaperSize(paperSize);
            pageSetup.setVerticalAlignment(PageVerticalAlignment.TOP);
            if (paperSize == PaperSize.A3) {
                pageSetup.setOrientation(Orientation.PORTRAIT);
            }
            doc.save(os, SaveFormat.PDF);//全面支持DOC, DOCX, OOXML, RTF HTML, OpenDocument, PDF, EPUB, XPS, SWF 相互转换
            long end = System.currentTimeMillis();
            log.info("共耗时：" + ((end - start) / 1000.0) + "秒"); //转化用时
        } catch (Exception e) {
            log.error("请求出错", e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            if (Objects.nonNull(os)) {
                os.flush();
                os.close();
            }
        }
        return file;
    }
}
