package com.qmth.teachcloud.mark.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.dto.mark.MarkSettingDto;
import com.qmth.teachcloud.common.bean.params.mark.setting.MarkPaperSettingConfig;
import com.qmth.teachcloud.common.bean.params.mark.setting.MarkPaperSettingList;
import com.qmth.teachcloud.common.bean.params.mark.setting.MarkPaperSettingParam;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.BasicExam;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.mark.MarkMode;
import com.qmth.teachcloud.common.enums.mark.MarkPaperStatus;
import com.qmth.teachcloud.common.enums.mark.SubjectiveStatus;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.service.BasicRoleDataPermissionService;
import com.qmth.teachcloud.common.service.TeachcloudCommonService;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.bean.archivescore.ArchiveScoreQuery;
import com.qmth.teachcloud.mark.bean.archivescore.ArchiveScoreVo;
import com.qmth.teachcloud.mark.bean.document.ArchivePaperQuery;
import com.qmth.teachcloud.mark.bean.document.ArchivePaperVo;
import com.qmth.teachcloud.mark.dto.mark.score.CheckScoreListDto;
import com.qmth.teachcloud.mark.dto.mark.score.MarkPaperPackageDto;
import com.qmth.teachcloud.mark.dto.mark.score.SettingDto;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.entity.MarkStudent;
import com.qmth.teachcloud.mark.entity.ScanPackage;
import com.qmth.teachcloud.mark.enums.OmrTaskStatus;
import com.qmth.teachcloud.mark.mapper.MarkPaperMapper;
import com.qmth.teachcloud.mark.service.*;
import com.qmth.teachcloud.mark.utils.Calculator;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 考试科目表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkPaperServiceImpl extends ServiceImpl<MarkPaperMapper, MarkPaper> implements MarkPaperService {

    @Resource
    private ScanPackageService scanPackageService;
    @Resource
    private ScanOmrTaskService scanOmrTaskService;
    @Resource
    private TeachcloudCommonService teachcloudCommonService;
    @Resource
    private BasicRoleDataPermissionService basicRoleDataPermissionService;
    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private MarkStudentService markStudentService;
    @Resource
    private MarkUserGroupService markUserGroupService;
    @Resource
    private MarkDocumentService markDocumentService;
    @Resource
    private BasicCourseService basicCourseService;


    @Override
    public IPage<MarkSettingDto> listPaperSetting(Long examId, Long courseId, String paperNumber, Boolean groupStatus, Integer pageNumber, Integer pageSize) {
        Page<MarkSettingDto> page = new Page<>(pageNumber, pageSize);
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), "/api/admin/mark/setting/list");
        IPage<MarkSettingDto> markSettingDtoIPage = this.baseMapper.listPaperSetting(page, sysUser.getSchoolId(), examId, courseId, paperNumber, groupStatus, dpr);
        for (MarkSettingDto record : markSettingDtoIPage.getRecords()) {
            if (Objects.nonNull(record.getMarkMode())) {
                record.setMarkModeDisplay(record.getMarkMode().getName());
            }
        }
        return markSettingDtoIPage;
    }

    @Override
    public MarkPaper getByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkPaper> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        return this.getOne(queryWrapper);
    }

    @Override
    public MarkPaper getByExamIdAndPaperNumberAndPaperType(Long examId, String paperNumber, String paperType) {
        QueryWrapper<MarkPaper> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<MarkPaper> lambda = queryWrapper.lambda();
        lambda.eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        if (StringUtils.isNotBlank(paperType)) {
            lambda.eq(MarkPaper::getPaperType, paperType);
        }
        return this.getOne(queryWrapper);
    }

    @Transactional
    @Override
    public void savePaperSetting(MarkPaper markPaper) {
        MarkPaper markPaperOld = this.getByExamIdAndPaperNumber(markPaper.getExamId(), markPaper.getPaperNumber());
        if (markPaperOld == null) {
            throw ExceptionResultEnum.ERROR.exception("评卷设置数据不存在");
        }
        // 评卷是否结束
        if (MarkPaperStatus.FINISH.equals(markPaperOld.getStatus())) {
            throw ExceptionResultEnum.ERROR.exception("科目已结束评卷，无法修改参数");
        }
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        LambdaUpdateWrapper<MarkPaper> lambda = updateWrapper.lambda();
        lambda.set(MarkPaper::getMarkMode, markPaper.getMarkMode())
                .set(MarkPaper::getForceMode, !MarkMode.UNLIMITED.equals(markPaper.getMarkMode()))
                .set(MarkPaper::getMarkStartTime, markPaper.getMarkStartTime())
                .set(MarkPaper::getMarkEndTime, markPaper.getMarkEndTime())
                .set(MarkPaper::getPassScore, markPaper.getPassScore())
                .set(MarkPaper::getExcellentScore, markPaper.getExcellentScore())
                .set(MarkPaper::getSheetView, markPaper.getSheetView())
                .set(MarkPaper::getShowObjectScore, markPaper.getShowObjectScore())
                .set(MarkPaper::getAutoScroll, markPaper.getAutoScroll())
                .set(MarkPaper::getOpenDoubleMarking, markPaper.getOpenDoubleMarking())
                .eq(MarkPaper::getExamId, markPaper.getExamId())
                .eq(MarkPaper::getPaperNumber, markPaper.getPaperNumber());
        if (StringUtils.isNotBlank(markPaper.getSheetConfig())) {
            lambda.set(MarkPaper::getSheetConfig, markPaper.getSheetConfig());
        }
        // 更新评卷员模式
        markUserGroupService.updateMode(markPaper.getExamId(), markPaper.getPaperNumber(), null, null, MarkMode.UNLIMITED.equals(markPaper.getMarkMode()) ? MarkMode.TRACK : markPaper.getMarkMode());
        this.update(updateWrapper);
    }

    @Transactional
    @Override
    public void saveBatchPaperSetting(MarkPaperSettingParam markPaperSettingParam) {
        List<MarkPaperSettingList> list = markPaperSettingParam.getList();
        if (CollectionUtils.isEmpty(list)) {
            throw ExceptionResultEnum.ERROR.exception("请选择要设置的数据");
        }

        MarkPaperSettingConfig config = markPaperSettingParam.getConfig();
        if (config == null) {
            throw ExceptionResultEnum.ERROR.exception("未找到参数值，请先设置");
        }

        for (MarkPaperSettingList markPaperSettingList : list) {
            MarkPaper markPaperOld = this.getByExamIdAndPaperNumber(markPaperSettingList.getExamId(), markPaperSettingList.getPaperNumber());
            if (markPaperOld == null) {
                throw ExceptionResultEnum.ERROR.exception("评卷设置数据不存在");
            }
            // 评卷是否结束
            if (MarkPaperStatus.FINISH.equals(markPaperOld.getStatus())) {
                throw ExceptionResultEnum.ERROR.exception("科目已结束评卷，无法修改参数");
            }

            UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
            LambdaUpdateWrapper<MarkPaper> lambda = updateWrapper.lambda();
            lambda.set(MarkPaper::getMarkMode, config.getMarkMode())
                    .set(MarkPaper::getForceMode, !MarkMode.UNLIMITED.equals(config.getMarkMode()))
                    .set(MarkPaper::getMarkStartTime, config.getMarkStartTime())
                    .set(MarkPaper::getMarkEndTime, config.getMarkEndTime())
                    .set(MarkPaper::getPassScore, config.getPassScore())
                    .set(MarkPaper::getExcellentScore, config.getExcellentScore())
                    .set(MarkPaper::getShowObjectScore, config.getShowObjectScore())
                    .set(MarkPaper::getAutoScroll, config.getAutoScroll())
                    .set(MarkPaper::getOpenDoubleMarking, config.getOpenDoubleMarking())
                    .eq(MarkPaper::getExamId, markPaperSettingList.getExamId())
                    .eq(MarkPaper::getPaperNumber, markPaperSettingList.getPaperNumber());
            // 更新评卷员模式
            markUserGroupService.updateMode(markPaperSettingList.getExamId(), markPaperSettingList.getPaperNumber(), null, null, MarkMode.UNLIMITED.equals(config.getMarkMode()) ? MarkMode.TRACK : config.getMarkMode());
            this.update(updateWrapper);
        }
    }

    @Override
    public Boolean finishPaper(BasicExam basicExam, List<String> paperNumbers, MarkPaperStatus status) {
        Long examId = basicExam.getId();
        for (String paperNumber : paperNumbers) {
            MarkPaper markPaper = this.getByExamIdAndPaperNumber(examId, paperNumber);
            if (MarkPaperStatus.FINISH.equals(status)) {
                BasicCourse basicCourse = basicCourseService.getById(markPaper.getCourseId());
                String courseInfo = String.format("%s[%s]，试卷编号%s，", basicCourse.getName(), basicCourse.getCode(), markPaper.getPaperNumber());
                // 主观题是否全部分组
                List<MarkQuestion> markQuestionObjectiveList = markQuestionService.listQuestionByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, null, true);
                if (CollectionUtils.isNotEmpty(markQuestionObjectiveList)) {
                    if (markQuestionObjectiveList.stream().filter(m -> StringUtils.isBlank(m.getAnswer())).count() > 0) {
                        throw ExceptionResultEnum.ERROR.exception(courseInfo + "客观题标答未设置，无法结束评卷");
                    }
                }

                // 主观题是否全部分组
                List<MarkQuestion> markQuestionSubjectiveList = markQuestionService.listQuestionByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, null, false);
                // 没有主观题，不校验考生评卷
                if (CollectionUtils.isNotEmpty(markQuestionSubjectiveList)) {
                    if (markQuestionSubjectiveList.stream().filter(m -> m.getGroupNumber() == null).count() > 0) {
                        throw ExceptionResultEnum.ERROR.exception(courseInfo + "主观题未全部分组，无法结束评卷");
                    }
                    // 未缺考、未违纪且已上传图片的考生全部评完
                    if (markStudentService.countByExamIdAndPaperNumberAndMarkStatus(examId, paperNumber, SubjectiveStatus.UNMARK) > 0) {
                        throw ExceptionResultEnum.ERROR.exception(courseInfo + "考生未全部评完，无法结束评卷");
                    }
                    // 未全部扫描，不能结束
                    if (markStudentService.countUnexistByExamIdAndPaperNumberAndPaperType(examId, paperNumber, markPaper.getPaperType()) > 0) {
                        throw ExceptionResultEnum.ERROR.exception(courseInfo + "有考生未扫描未核对，请在扫描客户端进行确认操作后再结束阅卷");
                    }
                    // 识别缺考未确认完，不能结束
                    if (markStudentService.countOmrAbsentStudent(examId, paperNumber, markPaper.getPaperType(), false) > 0) {
                        throw ExceptionResultEnum.ERROR.exception(courseInfo + "识别缺考记录未核对，请在扫描客户端进行确认操作后再结束阅卷");
                    }
                    // 人工绑定未做完，不能结束
                    if (markStudentService.getAssignedCount(examId, false, markPaper.getCourseId(), markPaper.getCoursePaperId(), MarkPaperStatus.FORMAL, null) > 0) {
                        throw ExceptionResultEnum.ERROR.exception(courseInfo + "人工绑定记录未核对，请在扫描客户端进行确认操作后再结束阅卷");
                    }
                    // 客观题检查未做完，不能结束
                    if (scanOmrTaskService.getFinishStudentCountByExamAndUserId(examId, markPaper.getCourseId(), markPaper.getCoursePaperId(), OmrTaskStatus.WAITING.name(), null) > 0) {
                        throw ExceptionResultEnum.ERROR.exception(courseInfo + "客观题存在识别嫌疑需要人工确认，请在扫描客户端进行确认操作后再结束阅卷");
                    }
                }

                // 结束评卷时，客观题统分
                List<MarkStudent> markStudentList = markStudentService.listByExamIdAndCoursePaperId(examId, markPaper.getCoursePaperId());
                for (MarkStudent markStudent : markStudentList) {
                    markStudentService.calculateObjectiveScore(markStudent);
                }

                // 结束评卷时，文档进行归档
                markDocumentService.initMarkDocument(basicExam, markPaper);
            }

            UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(MarkPaper::getStatus, status)
                    .eq(MarkPaper::getExamId, examId)
                    .eq(MarkPaper::getPaperNumber, paperNumber);
            this.update(updateWrapper);
        }

        return true;
    }

    /**
     * 校验试卷
     *
     * @param examId
     * @param paperNumber
     * @param teachClassName
     * @return
     */
    @Override
    public Boolean finishPaper(Long examId, String paperNumber, String teachClassName) {
        MarkPaper markPaper = this.getByExamIdAndPaperNumber(examId, paperNumber);
        Objects.requireNonNull(markPaper, "未找到评卷信息");
        BasicCourse basicCourse = basicCourseService.getById(markPaper.getCourseId());
        String courseInfo = String.format("%s[%s]，试卷编号%s，", basicCourse.getName(), basicCourse.getCode(), markPaper.getPaperNumber());
//        if (Objects.nonNull(markPaper.getStatus()) && markPaper.getStatus() != MarkPaperStatus.FINISH) {
//            throw ExceptionResultEnum.ERROR.exception(courseInfo + "评卷任务未完成，请完成评卷任务并检查成绩后再导出");
//        }
        // 主观题是否全部分组
        List<MarkQuestion> markQuestionObjectiveList = markQuestionService.listQuestionByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, null, true);
        if (CollectionUtils.isNotEmpty(markQuestionObjectiveList)) {
            if (markQuestionObjectiveList.stream().filter(m -> StringUtils.isBlank(m.getAnswer())).count() > 0) {
                throw ExceptionResultEnum.ERROR.exception(courseInfo + "客观题标答未设置，无法结束评卷");
            }
        }

        // 主观题是否全部分组
        List<MarkQuestion> markQuestionSubjectiveList = markQuestionService.listQuestionByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, null, false);
        // 没有主观题，不校验考生评卷
        if (CollectionUtils.isNotEmpty(markQuestionSubjectiveList)) {
            // 识别缺考未确认完，不能结束
            if (markStudentService.countOmrAbsentStudent(examId, paperNumber, markPaper.getPaperType(), false, teachClassName) > 0) {
                throw ExceptionResultEnum.ERROR.exception(courseInfo + "请登录扫描客户端完成缺考检查待处理任务");
            }
            // 人工绑定未做完，不能结束
            if (markStudentService.getAssignedCount(examId, false, markPaper.getCourseId(), markPaper.getCoursePaperId(), MarkPaperStatus.FORMAL, null, teachClassName) > 0) {
                throw ExceptionResultEnum.ERROR.exception(courseInfo + "请登录扫描客户端完成人工绑定检查待处理任务");
            }
            // 客观题检查未做完，不能结束
            if (scanOmrTaskService.getFinishStudentCountByExamAndUserId(examId, markPaper.getCourseId(), markPaper.getCoursePaperId(), OmrTaskStatus.WAITING.name(), null, teachClassName) > 0) {
                throw ExceptionResultEnum.ERROR.exception(courseInfo + "请登录扫描客户端完成客观题检查待处理任务");
            }
        }
        return true;
    }

    @Override
    public List<MarkPaper> listQualityMarkPaperByStatus(MarkPaperStatus status, int uploadCount) {
        return this.baseMapper.listQualityMarkPaperByStatus(status.name(), uploadCount);
    }

    @Override
    public int getCourseCount(Long examId, Long courseId, String coursePaperId, MarkPaperStatus status, DataPermissionRule dpr) {
        return baseMapper.getCourseCount(examId, courseId, coursePaperId, status.name(), dpr);
    }

    @Override
    public int getPaperNumberCount(Long examId, Long courseId, String coursePaperId, MarkPaperStatus status, DataPermissionRule dpr) {
        return baseMapper.getPaperNumberCount(examId, courseId, coursePaperId, status.name(), dpr);
    }

    @Override
    public IPage<CheckScoreListDto> listStudentScoreList(Long examId, Long courseId, String paperNumber, Integer pageNumber, Integer pageSize) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Page<CheckScoreListDto> page = new Page<>(pageNumber, pageSize);
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), ServletUtil.getRequest().getServletPath());
        return this.baseMapper.listStudentScoreList(page, examId, courseId, paperNumber, dpr);
    }

    @Override
    public void updateStatus(Long examId, String paperNumber, MarkPaperStatus newStatus, MarkPaperStatus currentStatus) {
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkPaper::getStatus, newStatus)
                .eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber)
                .eq(MarkPaper::getStatus, currentStatus);
        this.update(updateWrapper);
    }

    @Override
    public void updateUploadCount(Long examId, String paperNumber, int countUploaded) {
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkPaper::getUploadCount, countUploaded)
                .eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        this.update(updateWrapper);
    }

    @Override
    public IPage<MarkPaperPackageDto> listPackage(Long examId, String paperNumber, String packageCode, Integer pageNumber, Integer pageSize) {
        MarkPaper markPaper = this.getByExamIdAndPaperNumber(examId, paperNumber);
        Page<MarkPaperPackageDto> page = new Page<>(pageNumber, pageSize);
        IPage<MarkPaperPackageDto> markPaperPackageDtoIPage = this.baseMapper.listPackage(page, examId, markPaper.getCoursePaperId(), packageCode);
        for (MarkPaperPackageDto packageDto : markPaperPackageDtoIPage.getRecords()) {
            List<ScanPackage> scanPackageList = scanPackageService.listByExamIdAndCoursePaperIdAndPackageCode(examId, markPaper.getCoursePaperId(), packageDto.getPackageCode());
            List<String> urls = scanPackageList.stream().map(m -> teachcloudCommonService.filePreview(m.getPath())).collect(Collectors.toList());
            packageDto.setCount(urls.size());
            packageDto.setUrls(urls);
        }
        return markPaperPackageDtoIPage;
    }

    @Override
    public MarkPaper getByExamIdAndCoursePaperId(Long examId, String coursePaperId) {
        QueryWrapper<MarkPaper> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getCoursePaperId, coursePaperId);
        wrapper.last("LIMIT 1");
        return this.getOne(wrapper);
    }

    @Override
    public List<MarkPaper> listByExamId(Long examId, MarkPaperStatus status, DataPermissionRule dpr) {
        return this.baseMapper.listByExamId(examId, status.name(), dpr);
    }

    @Override
    public void updateGroupStatusByExamIdAndPaperNumber(boolean groupStatus, Long examId, String paperNumber) {
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkPaper::getGroupStatus, groupStatus)
                .eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        this.update(updateWrapper);
    }

    @Override
    public int countByPropositionTeacherId(boolean status) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, sysUser.getId(), "/api/admin/mark/setting/list");
        List<MarkSettingDto> markSettingDtoList = this.baseMapper.listPaperSetting(schoolId, null, null, null, status, dpr);
        return CollectionUtils.isEmpty(markSettingDtoList) ? 0 : markSettingDtoList.size();
    }

    @Override
    public void updateStudentCountByExamIdAndPaperNumberAndPaperType(Long examId, String paperNumber, String paperType) {
        long count = markStudentService.countByExamIdAndPaperNumber(examId, paperNumber, paperType);
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        LambdaUpdateWrapper<MarkPaper> lambda = updateWrapper.lambda();
        lambda.set(MarkPaper::getStudentCount, count)
                .eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        if (StringUtils.isNotBlank(paperType)) {
            lambda.eq(MarkPaper::getPaperType, paperType);
        }
        this.update(updateWrapper);
    }

    @Override
    public SettingDto getSetting(Long examId, String paperNumber) {
        SettingDto settingDto = new SettingDto();
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        settingDto.setExamId(examId);
        settingDto.setPaperNumber(paperNumber);
        settingDto.setUserName(sysUser.getRealName());
        MarkPaper markPaper = this.getByExamIdAndPaperNumber(examId, paperNumber);
        if (markPaper != null) {
            settingDto.setCourseCode(markPaper.getCourseCode());
            settingDto.setCourseName(markPaper.getCourseName());
            if (StringUtils.isNotBlank(markPaper.getPaperFilePath())) {
                settingDto.setPaperUrl(teachcloudCommonService.filePreview(markPaper.getPaperFilePath()));
            }
            if (StringUtils.isNotBlank(markPaper.getAnswerFilePath())) {
                settingDto.setAnswerUrl(teachcloudCommonService.filePreview(markPaper.getAnswerFilePath()));
            }
        }
        return settingDto;
    }

    @Override
    public IPage<ArchiveScoreVo> scoreList(ArchiveScoreQuery query) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Page<ArchiveScoreVo> page = new Page<>(query.getPageNumber(), query.getPageSize());
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), ServletUtil.getRequest().getServletPath());
        IPage<ArchiveScoreVo> ret = this.baseMapper.scoreList(page, sysUser.getSchoolId(), query, dpr);
        if (CollectionUtils.isNotEmpty(ret.getRecords())) {
            for (ArchiveScoreVo vo : ret.getRecords()) {
                double total = vo.getStudentCount() - vo.getAbsentCount();
                vo.setPassRate(Calculator.divide2String(Calculator.multiply(vo.getPassCount(), 100), total, 2));
                vo.setExcellentRate(Calculator.divide2String(Calculator.multiply(vo.getExcellentCount(), 100), total, 2));
                vo.setAvgScore(Calculator.round(vo.getAvgScore(), 2));
                vo.setMinScore(Calculator.round(vo.getMinScore(), 2));
                vo.setMaxScore(Calculator.round(vo.getMaxScore(), 2));
            }
        }
        return ret;
    }

    @Override
    public void deleteByExamIdAndPaperNumber(Long examId, String paperNumber) {
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        this.remove(updateWrapper);
    }

    @Override
    public void updateAbsentCount(Long examId, String paperNumber, int absentCount) {
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkPaper::getAbsentCount, absentCount)
                .eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        this.update(updateWrapper);
    }

    @Override
    public IPage<ArchivePaperVo> documentList(ArchivePaperQuery query) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Page<ArchiveScoreVo> page = new Page<>(query.getPageNumber(), query.getPageSize());
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), ServletUtil.getRequest().getServletPath());
        IPage<ArchivePaperVo> ret = this.baseMapper.documentList(page, query, dpr);
        return ret;
    }

    @Override
    public List<MarkPaper> findMarkPaperListByExamId(Long examId) {
        return this.baseMapper.findMarkPaperListByExamId(examId);
    }
}
