package com.qmth.teachcloud.obe.been.result.report.word;

import cn.hutool.core.date.DateUtil;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.obe.entity.TRBasicInfo;
import io.swagger.annotations.ApiModelProperty;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * @Description: 课程基本信息bean
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2024/1/31
 */
public class CourseBasicBean implements Serializable {

    @ApiModelProperty(name = "课程名称")
    String courseName;

    @ApiModelProperty(name = "课程编码")
    String courseCode;

    @ApiModelProperty(name = "课程英文名称")
    String courseEnName;

    @ApiModelProperty(name = "课程性质")
    String courseType;

    @ApiModelProperty(name = "学分")
    String credit;

    @ApiModelProperty(name = "学时")
    String period;

    @ApiModelProperty(name = "开课学院")
    String college;

    @ApiModelProperty(name = "开课专业")
    String profession;

    @ApiModelProperty(value = "开课时间")
    private String openTime;

    @ApiModelProperty(value = "授课对象")
    private String teachingObject;

    @ApiModelProperty(name = "评价人")
    String people;

    @ApiModelProperty(name = "评价时间")
    String date;

    @ApiModelProperty(name = "课程目标")
    String courseTarget;

    @ApiModelProperty(name = "课程目标数")
    Integer courseTargetNum;

    @ApiModelProperty(name = "平时作业权重")
    BigDecimal usualScoreWeight;

    @ApiModelProperty(name = "期末考试权重")
    BigDecimal finalScoreWeight;

    @ApiModelProperty(value = "参评人数")
    private Integer participantCount = 0;

    @ApiModelProperty(value = "任课老师")
    private String teacher;

    @ApiModelProperty(value = "达成情况")
    private String finishPoints;

    @ApiModelProperty(value = "课程支撑毕业要求达成情况评价")
    private String requirementPoints;

    @ApiModelProperty(value = "课程持续改进")
    private String courseSuggest;

    public CourseBasicBean() {

    }

    public CourseBasicBean(TRBasicInfo trBasicInfo, String courseTarget, Integer courseTargetNum) {
        this.courseName = trBasicInfo.getCourseName();
        this.courseCode = trBasicInfo.getCourseCode();
        this.courseType = trBasicInfo.getCourseType();
        this.credit = trBasicInfo.getCredit();
        this.period = trBasicInfo.getPeriod();
        this.people = trBasicInfo.getDirector();
        this.date = DateUtil.format(new Date(trBasicInfo.getCreateTime()), SystemConstant.DEFAULT_DATE_PATTERN);
        this.courseTarget = courseTarget;
        this.courseTargetNum = courseTargetNum;
        this.college = trBasicInfo.getCollege();
        this.profession = trBasicInfo.getProfession();
        this.openTime = trBasicInfo.getOpenTime();
        this.teachingObject = trBasicInfo.getTeachingObject();
        this.finishPoints = trBasicInfo.getFinishPoints();
        this.requirementPoints = trBasicInfo.getRequirementPoints();
        this.courseSuggest = trBasicInfo.getCourseSuggest();
        this.courseEnName = trBasicInfo.getCourseEnName();
        this.participantCount = trBasicInfo.getParticipantCount();
        this.teacher = trBasicInfo.getTeacher();
    }

    public String getFinishPoints() {
        return finishPoints;
    }

    public void setFinishPoints(String finishPoints) {
        this.finishPoints = finishPoints;
    }

    public String getRequirementPoints() {
        return requirementPoints;
    }

    public void setRequirementPoints(String requirementPoints) {
        this.requirementPoints = requirementPoints;
    }

    public String getCourseSuggest() {
        return courseSuggest;
    }

    public void setCourseSuggest(String courseSuggest) {
        this.courseSuggest = courseSuggest;
    }

    public Integer getParticipantCount() {
        return participantCount;
    }

    public void setParticipantCount(Integer participantCount) {
        this.participantCount = participantCount;
    }

    public String getTeacher() {
        return teacher;
    }

    public void setTeacher(String teacher) {
        this.teacher = teacher;
    }

    public String getOpenTime() {
        return openTime;
    }

    public void setOpenTime(String openTime) {
        this.openTime = openTime;
    }

    public String getTeachingObject() {
        return teachingObject;
    }

    public void setTeachingObject(String teachingObject) {
        this.teachingObject = teachingObject;
    }

    public String getCollege() {
        return college;
    }

    public void setCollege(String college) {
        this.college = college;
    }

    public String getProfession() {
        return profession;
    }

    public void setProfession(String profession) {
        this.profession = profession;
    }

    public String getPeriod() {
        return period;
    }

    public void setPeriod(String period) {
        this.period = period;
    }

    public BigDecimal getUsualScoreWeight() {
        return usualScoreWeight;
    }

    public void setUsualScoreWeight(BigDecimal usualScoreWeight) {
        this.usualScoreWeight = usualScoreWeight;
    }

    public BigDecimal getFinalScoreWeight() {
        return finalScoreWeight;
    }

    public void setFinalScoreWeight(BigDecimal finalScoreWeight) {
        this.finalScoreWeight = finalScoreWeight;
    }

    public String getCourseTarget() {
        return courseTarget;
    }

    public void setCourseTarget(String courseTarget) {
        this.courseTarget = courseTarget;
    }

    public Integer getCourseTargetNum() {
        return courseTargetNum;
    }

    public void setCourseTargetNum(Integer courseTargetNum) {
        this.courseTargetNum = courseTargetNum;
    }

    public String getCourseName() {
        return courseName;
    }

    public void setCourseName(String courseName) {
        this.courseName = courseName;
    }

    public String getCourseCode() {
        return courseCode;
    }

    public void setCourseCode(String courseCode) {
        this.courseCode = courseCode;
    }

    public String getCourseEnName() {
        return courseEnName;
    }

    public void setCourseEnName(String courseEnName) {
        this.courseEnName = courseEnName;
    }

    public String getCourseType() {
        return courseType;
    }

    public void setCourseType(String courseType) {
        this.courseType = courseType;
    }

    public String getCredit() {
        return credit;
    }

    public void setCredit(String credit) {
        this.credit = credit;
    }

    public String getPeople() {
        return people;
    }

    public void setPeople(String people) {
        this.people = people;
    }

    public String getDate() {
        return date;
    }

    public void setDate(String date) {
        this.date = date;
    }
}
