package com.qmth.teachcloud.obe.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.teachcloud.common.base.BaseEntity;
import com.qmth.teachcloud.obe.been.report.ReportCourseBasicInfoDto;
import com.qmth.teachcloud.obe.been.result.report.ReportResult;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import java.io.Serializable;
import java.math.BigDecimal;

/**
 * <p>
 * 报告基本情况表
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@ApiModel(value = "TRBasicInfo对象", description = "报告基本情况表")
public class TRBasicInfo extends BaseEntity implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "考试id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long examId;

    @ApiModelProperty(value = "培养方案id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long cultureProgramId;

    @ApiModelProperty(value = "课程id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long courseId;

    @ApiModelProperty(value = "课程编码")
    private String courseCode;

    @ApiModelProperty(value = "课程名称")
    private String courseName;

    @ApiModelProperty(value = "试卷编码")
    private String paperNumber;

    @ApiModelProperty(value = "课程性质")
    private String courseType;

    @ApiModelProperty(value = "学分")
    private String credit;

    @ApiModelProperty(value = "学时")
    private String period;

    @ApiModelProperty(value = "开课学院")
    private String college;

    @ApiModelProperty(value = "开课专业")
    private String profession;

    @ApiModelProperty(value = "考核方式")
    private String evaluationMode;

    @ApiModelProperty(value = "开课时间")
    private String openTime;

    @ApiModelProperty(value = "授课对象")
    private String teachingObject;

    @ApiModelProperty(value = "选课人数")
    private Integer selectionCount;

    @ApiModelProperty(value = "参评人数")
    private Integer participantCount = 0;

    @ApiModelProperty(value = "课程目标达成度期望值")
    private BigDecimal courseDegree = new BigDecimal(0);

    @ApiModelProperty(value = "任课老师")
    private String teacher;

    @ApiModelProperty(value = "评价责任人")
    private String director;

    @ApiModelProperty(value = "评价参与人")
    private String participant;

    @ApiModelProperty(value = "是否启用，0：停用，1：启用")
    private Boolean enable;

    @ApiModelProperty(value = "课程目标考核分布")
    private String courseEvaluationSpread;

    @ApiModelProperty(value = "课程考核成绩评价结果")
    private String courseEvaluationResult;

    @ApiModelProperty(value = "课程考核成绩评价明细结果")
    private String courseEvaluationResultDetail;

    @ApiModelProperty(value = "达成情况")
    private String finishPoints;

    @ApiModelProperty(value = "课程支撑毕业要求达成情况评价")
    private String requirementPoints;

    @ApiModelProperty(value = "课程持续改进")
    private String courseSuggest;

    @ApiModelProperty(name = "课程英文名称")
    private String courseEnName;

    @ApiModelProperty(value = "报告信息")
    @TableField(exist = false)
    private ReportResult reportResult;

    public TRBasicInfo() {

    }

    public TRBasicInfo(Long cultureProgramId, Long courseId, String courseCode, String courseName,
                       String paperNumber, String openTime,
                       Long userId, String finishPoints, String requirementPoints,
                       String courseSuggest, String courseEnName, Long examId,
                       ReportCourseBasicInfoDto reportCourseBasicInfoDto) {
        insertInfo(userId);
        this.cultureProgramId = cultureProgramId;
        this.courseId = courseId;
        this.courseCode = courseCode;
        this.courseName = courseName;
        this.paperNumber = paperNumber;
        this.openTime = openTime;
        this.teachingObject = reportCourseBasicInfoDto.getTeachingObject();
        this.enable = true;
        this.reportResult = new ReportResult();
        this.profession = reportCourseBasicInfoDto.getProfession();
        this.courseDegree = reportCourseBasicInfoDto.getCourseDegree();
        this.courseEnName = courseEnName;
        this.finishPoints = finishPoints;
        this.requirementPoints = requirementPoints;
        this.courseSuggest = courseSuggest;
        this.examId = examId;
        this.college = reportCourseBasicInfoDto.getCollege();
    }

    public String getCourseEnName() {
        return courseEnName;
    }

    public void setCourseEnName(String courseEnName) {
        this.courseEnName = courseEnName;
    }

    public String getFinishPoints() {
        return finishPoints;
    }

    public void setFinishPoints(String finishPoints) {
        this.finishPoints = finishPoints;
    }

    public String getRequirementPoints() {
        return requirementPoints;
    }

    public void setRequirementPoints(String requirementPoints) {
        this.requirementPoints = requirementPoints;
    }

    public String getCourseSuggest() {
        return courseSuggest;
    }

    public void setCourseSuggest(String courseSuggest) {
        this.courseSuggest = courseSuggest;
    }

    public String getCollege() {
        return college;
    }

    public void setCollege(String college) {
        this.college = college;
    }

    public String getProfession() {
        return profession;
    }

    public void setProfession(String profession) {
        this.profession = profession;
    }

    public TRBasicInfo(TRBasicInfo trBasicInfo, Long userId) {
        insertInfo(userId);
        this.setValue(trBasicInfo);
        this.enable = true;
    }

    public void updateInfo(TRBasicInfo trBasicInfo, ReportCourseBasicInfoDto reportCourseBasicInfoDto, Long userId) {
        this.setValue(trBasicInfo);
        this.college = reportCourseBasicInfoDto.getCollege();
        this.teachingObject = reportCourseBasicInfoDto.getTeachingObject();
        this.courseDegree = reportCourseBasicInfoDto.getCourseDegree();
        this.profession = reportCourseBasicInfoDto.getProfession();
        updateInfo(userId);
    }

    protected void setValue(TRBasicInfo trBasicInfo) {
        this.examId = trBasicInfo.getExamId();
        this.cultureProgramId = trBasicInfo.getCultureProgramId();
        this.courseId = trBasicInfo.getCourseId();
        this.courseCode = trBasicInfo.getCourseCode();
        this.courseName = trBasicInfo.getCourseName();
        this.paperNumber = trBasicInfo.getPaperNumber();
        this.courseType = trBasicInfo.getCourseType();
        this.credit = trBasicInfo.getCredit();
        this.period = trBasicInfo.getPeriod();
        this.evaluationMode = trBasicInfo.getEvaluationMode();
        this.openTime = trBasicInfo.getOpenTime();
        this.teachingObject = trBasicInfo.getTeachingObject();
        this.selectionCount = trBasicInfo.getSelectionCount();
        this.participantCount = trBasicInfo.getParticipantCount();
        this.courseDegree = trBasicInfo.getCourseDegree();
        this.teacher = trBasicInfo.getTeacher();
        this.director = trBasicInfo.getDirector();
        this.participant = trBasicInfo.getParticipant();
        this.courseEvaluationSpread = trBasicInfo.getCourseEvaluationSpread();
        this.courseEvaluationResult = trBasicInfo.getCourseEvaluationResult();
        this.courseEvaluationResultDetail = trBasicInfo.getCourseEvaluationResultDetail();
        this.profession = trBasicInfo.getProfession();
        this.courseEnName = trBasicInfo.getCourseEnName();
        this.college = trBasicInfo.getCollege();
        this.finishPoints = trBasicInfo.getFinishPoints();
        this.requirementPoints = trBasicInfo.getRequirementPoints();
        this.courseSuggest = trBasicInfo.getCourseSuggest();
    }

    public void setCalculate(TRBasicInfo trBasicInfo) {
        this.courseType = trBasicInfo.getCourseType();
        this.courseEnName = trBasicInfo.getCourseEnName();
        this.credit = trBasicInfo.getCredit();
        this.period = trBasicInfo.getPeriod();
        this.teachingObject = trBasicInfo.getTeachingObject();
        this.teacher = trBasicInfo.getTeacher();
        this.director = trBasicInfo.getDirector();
        this.finishPoints = trBasicInfo.getFinishPoints();
        this.requirementPoints = trBasicInfo.getRequirementPoints();
        this.courseSuggest = trBasicInfo.getCourseSuggest();
    }

    public void setBasicInfo(Long examId, String courseCode, String courseName, String paperNumber, String openTime,
                             Long cultureProgramId,
                             Long courseId, String courseEnName,
                             String finishPoints, String requirementPoints, String courseSuggest,
                             ReportCourseBasicInfoDto reportCourseBasicInfoDto) {
        this.examId = examId;
        this.courseCode = courseCode;
        this.courseName = courseName;
        this.paperNumber = paperNumber;
        this.openTime = openTime;
        this.teachingObject = reportCourseBasicInfoDto.getTeachingObject();
        this.teacher = reportCourseBasicInfoDto.getTeacher();
        this.enable = true;
        this.reportResult = new ReportResult();
        this.cultureProgramId = cultureProgramId;
        this.courseId = courseId;
        this.courseDegree = reportCourseBasicInfoDto.getCourseDegree();
        this.profession = reportCourseBasicInfoDto.getProfession();
        this.courseEnName = courseEnName;
        this.college = reportCourseBasicInfoDto.getCollege();
        this.finishPoints = finishPoints;
        this.requirementPoints = requirementPoints;
        this.courseSuggest = courseSuggest;
    }

    public Long getCultureProgramId() {
        return cultureProgramId;
    }

    public void setCultureProgramId(Long cultureProgramId) {
        this.cultureProgramId = cultureProgramId;
    }

    public Long getCourseId() {
        return courseId;
    }

    public void setCourseId(Long courseId) {
        this.courseId = courseId;
    }

    public ReportResult getReportResult() {
        return reportResult;
    }

    public void setReportResult(ReportResult reportResult) {
        this.reportResult = reportResult;
    }

    public String getCredit() {
        return credit;
    }

    public void setCredit(String credit) {
        this.credit = credit;
    }

    public String getPeriod() {
        return period;
    }

    public void setPeriod(String period) {
        this.period = period;
    }

    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public String getCourseCode() {
        return courseCode;
    }

    public void setCourseCode(String courseCode) {
        this.courseCode = courseCode;
    }

    public String getCourseName() {
        return courseName;
    }

    public void setCourseName(String courseName) {
        this.courseName = courseName;
    }

    public String getPaperNumber() {
        return paperNumber;
    }

    public void setPaperNumber(String paperNumber) {
        this.paperNumber = paperNumber;
    }

    public String getCourseType() {
        return courseType;
    }

    public void setCourseType(String courseType) {
        this.courseType = courseType;
    }

    public String getEvaluationMode() {
        return evaluationMode;
    }

    public void setEvaluationMode(String evaluationMode) {
        this.evaluationMode = evaluationMode;
    }

    public String getOpenTime() {
        return openTime;
    }

    public void setOpenTime(String openTime) {
        this.openTime = openTime;
    }

    public String getTeachingObject() {
        return teachingObject;
    }

    public void setTeachingObject(String teachingObject) {
        this.teachingObject = teachingObject;
    }

    public BigDecimal getCourseDegree() {
        return courseDegree;
    }

    public void setCourseDegree(BigDecimal courseDegree) {
        this.courseDegree = courseDegree;
    }

    public Integer getSelectionCount() {
        return selectionCount;
    }

    public void setSelectionCount(Integer selectionCount) {
        this.selectionCount = selectionCount;
    }

    public Integer getParticipantCount() {
        return participantCount;
    }

    public void setParticipantCount(Integer participantCount) {
        this.participantCount = participantCount;
    }

    public String getTeacher() {
        return teacher;
    }

    public void setTeacher(String teacher) {
        this.teacher = teacher;
    }

    public String getDirector() {
        return director;
    }

    public void setDirector(String director) {
        this.director = director;
    }

    public String getParticipant() {
        return participant;
    }

    public void setParticipant(String participant) {
        this.participant = participant;
    }

    public Boolean getEnable() {
        return enable;
    }

    public void setEnable(Boolean enable) {
        this.enable = enable;
    }

    public String getCourseEvaluationSpread() {
        return courseEvaluationSpread;
    }

    public void setCourseEvaluationSpread(String courseEvaluationSpread) {
        this.courseEvaluationSpread = courseEvaluationSpread;
    }

    public String getCourseEvaluationResult() {
        return courseEvaluationResult;
    }

    public void setCourseEvaluationResult(String courseEvaluationResult) {
        this.courseEvaluationResult = courseEvaluationResult;
    }

    public String getCourseEvaluationResultDetail() {
        return courseEvaluationResultDetail;
    }

    public void setCourseEvaluationResultDetail(String courseEvaluationResultDetail) {
        this.courseEvaluationResultDetail = courseEvaluationResultDetail;
    }
}
