package com.qmth.teachcloud.obe.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.deepoove.poi.data.ChartMultiSeriesRenderData;
import com.deepoove.poi.data.TableRenderData;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.obe.been.dto.CourseWeightDto;
import com.qmth.teachcloud.obe.been.report.CourseTargetWordDto;
import com.qmth.teachcloud.obe.been.report.ReportCourseBasicInfoDto;
import com.qmth.teachcloud.obe.been.report.ReportCourseEvaluationResultDto;
import com.qmth.teachcloud.obe.been.report.ReportScoreViewDto;
import com.qmth.teachcloud.obe.been.result.ObeCourseWeightResult;
import com.qmth.teachcloud.obe.been.result.report.PaperStructDimensionResult;
import com.qmth.teachcloud.obe.been.result.report.word.CourseReportBean;
import com.qmth.teachcloud.obe.entity.ObeCourseOutline;
import com.qmth.teachcloud.obe.entity.TRBasicInfo;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 报告基本情况表 服务类
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
public interface TRBasicInfoService extends IService<TRBasicInfo> {

    /**
     * 根据考试id/课程编码/试卷编码查询数据库数据
     *
     * @param cultureProgramId cultureProgramId
     * @param courseId         courseId
     * @param paperNumber      paperNumber
     * @return
     */
    public TRBasicInfo queryBasicInfo(Long cultureProgramId, Long courseId, String paperNumber);

    /**
     * 获取课程目标考核分布-分数图
     *
     * @param trBasicInfo
     * @param markPaper
     * @return
     */
    public ReportScoreViewDto getScoreList(TRBasicInfo trBasicInfo, MarkPaper markPaper);

    /**
     * 获取计算参数
     *
     * @param trBasicInfo
     * @param paperStructDimensionResultList
     * @param obeCourseWeightResult
     * @return
     */
    public Map<String, Object> getReportCalculateParams(TRBasicInfo trBasicInfo, List<PaperStructDimensionResult> paperStructDimensionResultList,
                                                        ObeCourseWeightResult obeCourseWeightResult);

    /**
     * 计算课程考核成绩评价结果
     *
     * @param trBasicInfo
     * @param paramsMap
     * @return
     */
    public ReportCourseEvaluationResultDto reportCalculateCourseEvaluationResult(TRBasicInfo trBasicInfo, Map<String, Object> paramsMap);

    /**
     * 获取报告
     *
     * @param trBasicInfo
     * @param markPaper
     * @param userId
     * @param obeCourseWeightResult
     * @param cultureProgramId
     * @param courseId
     * @return
     */
    public TRBasicInfo getReportView(TRBasicInfo trBasicInfo, MarkPaper markPaper, Long userId, ObeCourseWeightResult obeCourseWeightResult, Long cultureProgramId, Long courseId);

    /**
     * 构建word动态表格1-课程目标与毕业要求指标点的对应关系
     *
     * @param courseTargetWordDtoList
     * @return
     */
    public TableRenderData buildWordTable1(List<CourseTargetWordDto> courseTargetWordDtoList);

    /**
     * 构建word动态表格2-课程目标达成考核/评价环节及目标分值
     *
     * @param courseTargetWordDtoList
     * @return
     */
    public TableRenderData buildWordTable2(List<CourseTargetWordDto> courseTargetWordDtoList);

    /**
     * 构建word动态表格5-各课程目标考核样本数据详情
     *
     * @param trBasicInfo
     * @param courseReportBean
     * @return
     */
    public TableRenderData buildWordTable5(TRBasicInfo trBasicInfo, CourseReportBean courseReportBean);

    /**
     * 构建word动态表格6
     *
     * @param map
     * @return
     */
    public TableRenderData buildWordTable6(Map<String, Object> map);

    /**
     * 构建word图形表格1
     *
     * @param map
     * @return
     */
    public ChartMultiSeriesRenderData buildCourseTargetColumnDiagram1(Map<String, Object> map);

    /**
     * 构建word图形表格2
     *
     * @param map
     * @return
     */
    public ChartMultiSeriesRenderData buildCourseTargetColumnDiagram2(Map<String, Object> map);

    /**
     * 根据考试id/课程编码/试卷编码删除数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @return
     */
    public Boolean remove(Long cultureProgramId, Long courseId, String paperNumber);

    /**
     * 查找课程评价值
     *
     * @param obeCourseOutlineId
     * @return
     */
    public ObeCourseWeightResult findCourseWeightResultRmi(Long obeCourseOutlineId);

    /**
     * 查找课程评价值
     *
     * @param obeCourseOutlineId
     * @return
     */
    public List<CourseWeightDto> findCourseWeightDtoRmi(Long obeCourseOutlineId);

    /**
     * 清除报告数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param clearDimension
     */
    public void clearReportData(Long cultureProgramId, Long courseId, String paperNumber, boolean clearDimension);

    /**
     * 报表基础信息
     *
     * @param trBasicInfo
     * @param obeCourseOutline
     * @param markPaper
     * @param obeCourseWeightResult
     * @return
     */
    public ReportCourseBasicInfoDto setBasicInfo(TRBasicInfo trBasicInfo, ObeCourseOutline obeCourseOutline, MarkPaper markPaper, ObeCourseWeightResult obeCourseWeightResult);
}
