package com.qmth.teachcloud.obe.service.impl;

import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.result.DictionaryResult;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.BasicProfessional;
import com.qmth.teachcloud.common.service.BasicRoleDataPermissionService;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.obe.entity.ObeCultureProgram;
import com.qmth.teachcloud.obe.mapper.ObeConditionMapper;
import com.qmth.teachcloud.obe.service.ObeConditionService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 查询条件 服务实现类
 * </p>
 */
@Service
public class ObeConditionServiceImpl implements ObeConditionService {

    @Resource
    BasicRoleDataPermissionService basicRoleDataPermissionService;
    @Resource
    ObeConditionMapper obeConditionMapper;

    @Override
    public List<BasicProfessional> listProfessional() {
        Long privilegeId = ServletUtil.getCurrentPrivilegeId();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(privilegeId);
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        // 未找到权限菜单，默认为本人权限
        return obeConditionMapper.listProfessional(schoolId, dpr);
    }

    @Override
    public List<ObeCultureProgram> listCultureProgram(Long professionalId) {
        List<ObeCultureProgram> result = new ArrayList<>();
        Long privilegeId = ServletUtil.getCurrentPrivilegeId();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(privilegeId);
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        String privilegeIdString = privilegeId == null ? null : String.valueOf(privilegeId);
        // 课程大纲管理、课程考核设置
        if ("3000".equals(privilegeIdString) || "3001".equals(privilegeIdString) || "2080".equals(privilegeIdString)) {
            result = obeConditionMapper.listCultureProgram(schoolId, professionalId, dpr);
        }
        // 课程目标达成度统计、成绩管理
        else if ("2002".equals(privilegeIdString) || "2001".equals(privilegeIdString)) {
            result = obeConditionMapper.listCultureProgramForDegree(schoolId, dpr);
        }
        return result;
    }

    @Override
    public List<DictionaryResult> listOutlineSemester(Long professionalId, Long cultureProgramId) {
        Long privilegeId = ServletUtil.getCurrentPrivilegeId();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(privilegeId);
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());

        return obeConditionMapper.listOutlineSemester(schoolId, professionalId, cultureProgramId, dpr);
    }

    @Override
    public List<BasicCourse> listCultureProgramCourse(Long professionalId, Long cultureProgramId, Long semesterId) {
        Long privilegeId = ServletUtil.getCurrentPrivilegeId();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(privilegeId);
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        return obeConditionMapper.listCultureProgramCourse(schoolId, professionalId, cultureProgramId, semesterId, dpr);
    }
}
