package com.qmth.teachcloud.obe.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.bean.result.ExcelResult;
import com.qmth.teachcloud.common.bean.tiku.TikuCourseProperty;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.kit.TikuUtils;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.util.ConvertUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.obe.been.dto.CourseDimensionImportDto;
import com.qmth.teachcloud.obe.been.dto.CourseDimensionMd5Dto;
import com.qmth.teachcloud.obe.been.result.CourseDimensionTree;
import com.qmth.teachcloud.obe.entity.ObeCourseDimension;
import com.qmth.teachcloud.obe.entity.ObeCourseOutline;
import com.qmth.teachcloud.obe.enums.CourseDimensionCodeEnum;
import com.qmth.teachcloud.obe.enums.CourseDimensionSourceEnum;
import com.qmth.teachcloud.obe.enums.CourseSettingTypeEnum;
import com.qmth.teachcloud.obe.mapper.ObeCourseDimensionMapper;
import com.qmth.teachcloud.obe.service.ObeCourseDimensionService;
import com.qmth.teachcloud.obe.service.ObeCourseOutlineService;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 课程知识点表 服务实现类
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-02-27
 */
@Service
public class ObeCourseDimensionServiceImpl extends ServiceImpl<ObeCourseDimensionMapper, ObeCourseDimension> implements
        ObeCourseDimensionService {

    @Resource
    ObeCourseOutlineService obeCourseOutlineService;

    @Resource
    TikuUtils tikuUtils;

    @Resource
    BasicCourseService basicCourseService;

    @Transactional
    @Override
    public void importCourseDimension(SysUser requestUser, MultipartFile file, Long obeCourseOutlineId) throws Exception {
        Long requestUserId = requestUser.getId();
        Long schoolId = requestUser.getSchoolId();

        ObeCourseOutline courseOutline = obeCourseOutlineService.getById(obeCourseOutlineId);
        if (Objects.isNull(courseOutline)) {
            throw ExceptionResultEnum.ERROR.exception("课程大纲不存在");
        }

        ExcelResult<CourseDimensionImportDto> excelResult = ConvertUtil.analyzeExcel(file.getInputStream(),
                CourseDimensionImportDto.class, false, 1);
        List<CourseDimensionImportDto> courseDimensionImportDtoList = excelResult.getDatasource();
        boolean success = excelResult.isSuccess();
        if (!success) {
            throw ExceptionResultEnum.ERROR.exception(excelResult.getErrorMsg());
        }

        // 处理要新增的考察点
        Map<String, List<String>> dimensionMap = new LinkedHashMap<>();
        for (CourseDimensionImportDto courseDimensionImportDto : courseDimensionImportDtoList) {
            String firstDimensionName = courseDimensionImportDto.getFirstDimensionName();
            if (!SystemConstant.strNotNull(firstDimensionName)) {
                throw ExceptionResultEnum.ERROR.exception("excel中存在没有填写的一级知识点");
            }
            String secondDimensionName = courseDimensionImportDto.getSecondDimensionName();

            if (dimensionMap.containsKey(firstDimensionName)) {
                if (SystemConstant.strNotNull(secondDimensionName)) {
                    List<String> sl = dimensionMap.get(firstDimensionName);
                    if (!sl.contains(secondDimensionName)) {
                        sl.add(secondDimensionName);
                        dimensionMap.put(firstDimensionName, sl);
                    }
                }
            } else {
                List<String> sl = new ArrayList<>();
                if (SystemConstant.strNotNull(secondDimensionName)) {
                    sl.add(secondDimensionName);
                }
                dimensionMap.put(firstDimensionName, sl);
            }
        }
        ObeCourseDimension temp = new ObeCourseDimension();
        temp.setObeCourseOutlineId(obeCourseOutlineId);
        temp.setSchoolId(schoolId);
        temp.setSource(CourseDimensionSourceEnum.IMPORT);

        List<ObeCourseDimension> willSaveDimensions = new ArrayList<>();

        AtomicInteger firstNumber = new AtomicInteger(0);
        for (String firstDimensionName : dimensionMap.keySet()) {
            ObeCourseDimension first = new ObeCourseDimension();
            int index = firstNumber.incrementAndGet();
            BeanUtils.copyProperties(temp, first);
            String firstCode = CourseDimensionCodeEnum.getCodeByIndex(index);
            first.setCode(firstCode);
            first.setName(firstDimensionName);
            first.setNumber(index);
            first.insertInfo(requestUserId);
            first.setParentId(0L);
            willSaveDimensions.add(first);

            List<String> secondDimensionList = dimensionMap.get(firstDimensionName);
            Long firstId = first.getId();
            for (int i = 0; i < secondDimensionList.size(); i++) {
                ObeCourseDimension second = new ObeCourseDimension();
                int secondIndex = i + 1;
                BeanUtils.copyProperties(temp, second);
                String secondCode = firstCode + secondIndex;
                second.setCode(secondCode);
                second.setName(secondDimensionList.get(i));
                second.setNumber(secondIndex);
                second.insertInfo(requestUserId);
                second.setParentId(firstId);
                willSaveDimensions.add(second);
            }
        }
        List<ObeCourseDimension> dbList = this.list(new QueryWrapper<ObeCourseDimension>().lambda().eq(ObeCourseDimension::getObeCourseOutlineId, obeCourseOutlineId));
        String dbMd5 = this.buildCourseDimensionMd5(dbList);
        String willAddMd5 = this.buildCourseDimensionMd5(willSaveDimensions);
        if (!willAddMd5.equals(dbMd5)) {
            // 删除课程设置
            obeCourseOutlineService.clearCourseSetting(obeCourseOutlineId, CourseSettingTypeEnum.COURSE_DIMENSION);
            this.saveBatch(willSaveDimensions);
        }
    }

    @Transactional
    @Override
    public void syncTikuCourseProperties(Long obeCourseOutlineId) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        Long requestUserId = requestUser.getId();
        ObeCourseOutline courseOutline = obeCourseOutlineService.getById(obeCourseOutlineId);
        if (Objects.isNull(courseOutline)) {
            throw ExceptionResultEnum.ERROR.exception("课程大纲不存在");
        }
        Long schoolId = courseOutline.getSchoolId();
        Long courseId = courseOutline.getCourseId();
        BasicCourse basicCourse = basicCourseService.getById(courseId);
        if (Objects.isNull(basicCourse)) {
            throw ExceptionResultEnum.ERROR.exception("基础课程不存在");
        }

        List<ObeCourseDimension> obeCourseDimensionList = this.list(new QueryWrapper<ObeCourseDimension>().lambda().eq(ObeCourseDimension::getObeCourseOutlineId, obeCourseOutlineId));

        if (obeCourseDimensionList.stream().noneMatch(e -> CourseDimensionSourceEnum.IMPORT.equals(e.getSource()))) {
            // 表里没有导入的知识点记录,从题库获取
            List<TikuCourseProperty> tikuDimensionList = tikuUtils.listCourseProperty(courseOutline.getSchoolId(),
                    basicCourse.getCode());
            if (CollectionUtils.isNotEmpty(tikuDimensionList)) {
                ObeCourseDimension temp = new ObeCourseDimension();
                temp.setObeCourseOutlineId(obeCourseOutlineId);
                temp.setSchoolId(schoolId);
                temp.setSource(CourseDimensionSourceEnum.UNION_QUESTION);
                List<ObeCourseDimension> willAddList = new ArrayList<>();

                for (TikuCourseProperty first : tikuDimensionList) {
                    Long fParentId = first.getParentId();
                    if (fParentId > 0) {
                        throw ExceptionResultEnum.ERROR.exception("题库一级知识点父id异常");
                    }
                    String fName = first.getName();
                    String fCode = first.getCode();
                    int fNumber = Integer.parseInt(first.getNumber());

                    ObeCourseDimension willAddFirst = new ObeCourseDimension();
                    BeanUtils.copyProperties(temp, willAddFirst);
                    willAddFirst.setCode(fCode);
                    willAddFirst.setName(fName);
                    willAddFirst.setNumber(fNumber);
                    willAddFirst.insertInfo(requestUserId);
                    willAddFirst.setParentId(0L);
                    willAddList.add(willAddFirst);

                    List<TikuCourseProperty> secondList = first.getPropertyList();
                    for (TikuCourseProperty second : secondList) {
                        Long sParentId = willAddFirst.getId();
                        String sName = second.getName();
                        String sCode = second.getCode();
                        int sNumber = Integer.parseInt(second.getNumber());

                        ObeCourseDimension willAddSecond = new ObeCourseDimension();
                        BeanUtils.copyProperties(temp, willAddSecond);
                        willAddSecond.setCode(sCode);
                        willAddSecond.setName(sName);
                        willAddSecond.setNumber(sNumber);
                        willAddSecond.insertInfo(requestUserId);
                        willAddSecond.setParentId(sParentId);
                        willAddList.add(willAddSecond);
                    }
                }

                if (CollectionUtils.isNotEmpty(willAddList)) {
                    String dbMd5 = this.buildCourseDimensionMd5(obeCourseDimensionList.stream().filter(e -> CourseDimensionSourceEnum.UNION_QUESTION.equals(e.getSource()))
                            .collect(Collectors.toList()));
                    String willAddMd5 = this.buildCourseDimensionMd5(willAddList);
                    if (!willAddMd5.equals(dbMd5)) {
                        // 如果题库知识点和之前的题库知识点不一致，删除原先的并新增
                        obeCourseOutlineService.clearCourseSetting(obeCourseOutlineId, CourseSettingTypeEnum.COURSE_DIMENSION);
                        this.saveBatch(willAddList);
                    }
                }
            }
        }
    }

    @Override
    public List<CourseDimensionTree> findDimensionTree(Long obeCourseOutlineId) {
        ObeCourseOutline courseOutline = obeCourseOutlineService.getById(obeCourseOutlineId);
        if (Objects.isNull(courseOutline)) {
            throw ExceptionResultEnum.ERROR.exception("课程大纲不存在");
        }
        List<ObeCourseDimension> obeCourseDimensionList = this.list(new QueryWrapper<ObeCourseDimension>().lambda().eq(ObeCourseDimension::getObeCourseOutlineId, obeCourseOutlineId));

        List<ObeCourseDimension> firstDimensionList = obeCourseDimensionList.stream().filter(e -> e.getParentId() == 0).sorted(Comparator.comparingInt(ObeCourseDimension::getNumber)).collect(Collectors.toList());

        List<CourseDimensionTree> treeList = new ArrayList<>();
        for (ObeCourseDimension first : firstDimensionList) {
            Long id = first.getId();
            String code = first.getCode();
            String name = first.getName();
            List<CourseDimensionTree> children = new ArrayList<>();

            List<ObeCourseDimension> secondDimensionList = obeCourseDimensionList.stream().filter(e -> id.equals(e.getParentId()))
                    .sorted(Comparator.comparingInt(ObeCourseDimension::getNumber)).collect(Collectors.toList());
            for (ObeCourseDimension child : secondDimensionList) {
                Long childId = child.getId();
                String childCode = child.getCode();
                String childName = child.getName();

                CourseDimensionTree tree = new CourseDimensionTree(childId, childCode, childName, new ArrayList<>());
                children.add(tree);
            }
            CourseDimensionTree tree = new CourseDimensionTree(id, code, name, children);
            treeList.add(tree);
        }
        return treeList;
    }

    /**
     * 构建知识点md5
     *
     * @param obeCourseDimensionList 课程知识点集合
     * @return MD5
     */
    private String buildCourseDimensionMd5(List<ObeCourseDimension> obeCourseDimensionList) {
        List<CourseDimensionMd5Dto> md5List = obeCourseDimensionList.stream().flatMap(e -> {
            CourseDimensionMd5Dto dto = new CourseDimensionMd5Dto();
            dto.setCode(e.getCode());
            dto.setName(e.getName());
            dto.setNumber(e.getNumber());
            dto.setSource(e.getSource());
            return Stream.of(dto);
        }).sorted(Comparator.comparing(CourseDimensionMd5Dto::getCode)).collect(Collectors.toList());
        System.out.println(JSON.toJSONString(md5List));
        return DigestUtils.md5Hex(JSON.toJSONString(md5List));
    }
}
