package com.qmth.teachcloud.obe.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.obe.been.obe.CourseRequirementMatrixDto;
import com.qmth.teachcloud.obe.been.obe.MatrixRequirementDto;
import com.qmth.teachcloud.obe.been.obe.MatrixSubRequirementDto;
import com.qmth.teachcloud.obe.been.report.*;
import com.qmth.teachcloud.obe.been.result.FinalScoreResult;
import com.qmth.teachcloud.obe.been.result.report.studentRequirement.*;
import com.qmth.teachcloud.obe.entity.ObeCourseRequirementReport;
import com.qmth.teachcloud.obe.entity.ObeCourseTarget;
import com.qmth.teachcloud.obe.entity.TRBasicInfo;
import com.qmth.teachcloud.obe.entity.TRExamStudent;
import com.qmth.teachcloud.obe.mapper.TRExamStudentMapper;
import com.qmth.teachcloud.obe.service.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * <p>
 * 报告考生表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@Service
public class TRExamStudentServiceImpl extends ServiceImpl<TRExamStudentMapper, TRExamStudent> implements TRExamStudentService {
    private final static Logger log = LoggerFactory.getLogger(TRExamStudentServiceImpl.class);

    @Resource
    TRExamStudentService trExamStudentService;

    @Resource
    TRBasicInfoService trBasicInfoService;

    @Resource
    private ObeCourseTargetService obeCourseTargetService;

    @Resource
    private ObeCourseRequirementMatrixService obeCourseRequirementMatrixService;

    @Resource
    private ObeCourseRequirementReportService obeCourseRequirementReportService;

    /**
     * 获取考生详细报告
     *
     * @param trBasicInfo
     * @param paramsMap
     * @return
     */
    @Override
    @Transactional
    public List<ReportExamStudentDto> getExamStudentReportDetail(TRBasicInfo trBasicInfo, Map<String, Object> paramsMap) {
        List<ReportExamStudentDto> examStudentList = (List<ReportExamStudentDto>) paramsMap.get("examStudentList");
        Map<String, FinalScoreResult> finalScoreResultMap = (Map<String, FinalScoreResult>) paramsMap.get("finalScoreResultMap");
        Map<String, Map<Long, BigDecimal>> finalScoreExamStudentTargetMap = (Map<String, Map<Long, BigDecimal>>) paramsMap.get("finalScoreExamStudentTargetMap");
        Map<Long, CourseTargetWordDto> targetWordMap = (Map<Long, CourseTargetWordDto>) paramsMap.get("targetWordMap");
        if (!CollectionUtils.isEmpty(examStudentList)) {
            List<TRExamStudent> trExamStudentList = new ArrayList<>(examStudentList.size());
            Map<Long, ReportExamStudentTargetDto> reportExamStudentTargetDtoMap = new LinkedHashMap<>();
            for (ReportExamStudentDto reportExamStudentDto : examStudentList) {
                Map<Long, BigDecimal> map = finalScoreExamStudentTargetMap.get(reportExamStudentDto.getStudentCode());
                FinalScoreResult finalScoreResult = finalScoreResultMap.get(reportExamStudentDto.getStudentCode());
                ReportExamStudentUsualScoreDto reportExamStudentUsualScoreSumDto = finalScoreResult.getUsualScoreDto();
                map.forEach((k, v) -> {
                    List<ReportExamStudentTargetDto> targetList = CollectionUtils.isEmpty(reportExamStudentDto.getTargetList()) ? new ArrayList<>() : reportExamStudentDto.getTargetList();
                    if (targetWordMap.containsKey(k)) {
                        List<ReportExamStudentUsualScoreObjDto> scoreObjDtoList = reportExamStudentUsualScoreSumDto.getScoreMap().get(k);
                        CourseTargetWordDto courseTargetWordDto = targetWordMap.get(k);

                        ReportExamStudentFinalScoreDto reportExamStudentFinalScoreDto = Objects.nonNull(courseTargetWordDto.getFinalScoreDto()) ? new ReportExamStudentFinalScoreDto(courseTargetWordDto.getFinalScoreDto(), finalScoreResult.getFinalScoreTarget().get(k)) : null;
                        ReportExamStudentUsualScoreDto reportExamStudentUsualScoreDto = new ReportExamStudentUsualScoreDto(scoreObjDtoList);

                        BigDecimal targetMatrixSumScore = new BigDecimal(0);
                        AtomicReference<BigDecimal> matrixDegree = new AtomicReference<>(new BigDecimal(0));
                        if (!reportExamStudentTargetDtoMap.containsKey(k)) {
                            BigDecimal targetMatrixScore = new BigDecimal(0);
                            if (Objects.nonNull(courseTargetWordDto.getFinalScoreDto())) {
                                //卷面分/目标卷面总分*权重
                                if (courseTargetWordDto.getFinalScoreDto().getFinalScoreTargetSumScore().doubleValue() > 0) {
                                    targetMatrixScore = v.divide(courseTargetWordDto.getFinalScoreDto().getFinalScoreTargetSumScore(), 4, BigDecimal.ROUND_HALF_UP).multiply(courseTargetWordDto.getFinalScoreDto().getFinalScoreSumWeight()).setScale(2, BigDecimal.ROUND_HALF_UP);
                                    BigDecimal targetEvaluationValue = v.divide(courseTargetWordDto.getFinalScoreDto().getFinalScoreTargetSumScore(), 4, BigDecimal.ROUND_HALF_UP).multiply(courseTargetWordDto.getFinalScoreDto().getFinalScoreSumWeight().divide(courseTargetWordDto.getTargetSumWeight(), 4, BigDecimal.ROUND_HALF_UP)).setScale(2, BigDecimal.ROUND_HALF_UP);
                                    matrixDegree.set(matrixDegree.get().add(targetEvaluationValue));
                                }
                            }
                            targetMatrixSumScore = targetMatrixSumScore.add(targetMatrixScore);
                            reportExamStudentDto.setScore(reportExamStudentDto.getScore().add(targetMatrixScore));

                            if (Objects.nonNull(reportExamStudentFinalScoreDto)) {
                                reportExamStudentFinalScoreDto.setTargetMatrixSumScore(targetMatrixSumScore);
                            }
                            ReportExamStudentTargetDto reportExamStudentTargetDto = null;
                            //平时作业总和+累加
                            if (!CollectionUtils.isEmpty(scoreObjDtoList)) {
                                for (ReportExamStudentUsualScoreObjDto r : scoreObjDtoList) {
                                    BigDecimal targetEvaluationValue = r.getScore().divide(r.getTargetScore(), 4, BigDecimal.ROUND_HALF_UP).multiply(r.getTargetWeight().divide(courseTargetWordDto.getTargetSumWeight(), 4, BigDecimal.ROUND_HALF_UP)).setScale(2, BigDecimal.ROUND_HALF_UP);
                                    matrixDegree.set(matrixDegree.get().add(targetEvaluationValue));
                                }
                                Double usualMatrixSumScore = scoreObjDtoList.stream().mapToDouble(s -> s.getMatrixScore().doubleValue()).sum();
                                reportExamStudentDto.setScore(reportExamStudentDto.getScore().add(new BigDecimal(usualMatrixSumScore).setScale(2, BigDecimal.ROUND_HALF_UP)));
                                ReportExamStudentFinalScoreDto reportExamStudentFinalScoreTempDto = null;
                                if (Objects.nonNull(reportExamStudentFinalScoreDto)) {
                                    reportExamStudentFinalScoreTempDto = new ReportExamStudentFinalScoreDto(courseTargetWordDto.getFinalScoreDto(), targetMatrixSumScore, finalScoreResult.getFinalScoreTarget().get(k));
                                }
                                reportExamStudentTargetDto = new ReportExamStudentTargetDto(courseTargetWordDto, reportExamStudentFinalScoreTempDto, new ReportExamStudentUsualScoreDto(scoreObjDtoList.stream().map(ReportExamStudentUsualScoreObjDto::new).collect(Collectors.toList())), matrixDegree.get());
                            } else {
                                reportExamStudentTargetDto = new ReportExamStudentTargetDto(courseTargetWordDto, reportExamStudentFinalScoreDto, null, matrixDegree.get());
                            }
                            reportExamStudentTargetDtoMap.put(k, reportExamStudentTargetDto);
                        } else {
                            ReportExamStudentTargetDto reportExamStudentTargetDto = reportExamStudentTargetDtoMap.get(k);
                            ReportExamStudentFinalScoreDto reportExamStudentFinalScoreAvgDto = null;
                            BigDecimal targetMatrixScore = new BigDecimal(0);
                            if (Objects.nonNull(reportExamStudentTargetDto.getFinalScore())) {
                                reportExamStudentFinalScoreAvgDto = reportExamStudentTargetDto.getFinalScore();
                                if (courseTargetWordDto.getFinalScoreDto().getFinalScoreTargetSumScore().doubleValue() > 0) {
                                    targetMatrixScore = v.divide(courseTargetWordDto.getFinalScoreDto().getFinalScoreTargetSumScore(), 4, BigDecimal.ROUND_HALF_UP).multiply(courseTargetWordDto.getFinalScoreDto().getFinalScoreSumWeight()).setScale(2, BigDecimal.ROUND_HALF_UP);
                                    BigDecimal targetEvaluationValue = v.divide(courseTargetWordDto.getFinalScoreDto().getFinalScoreTargetSumScore(), 4, BigDecimal.ROUND_HALF_UP).multiply(courseTargetWordDto.getFinalScoreDto().getFinalScoreSumWeight().divide(courseTargetWordDto.getTargetSumWeight(), 4, BigDecimal.ROUND_HALF_UP)).setScale(2, BigDecimal.ROUND_HALF_UP);
                                    matrixDegree.set(matrixDegree.get().add(targetEvaluationValue));
                                }
                                reportExamStudentTargetDto.setMatrixDegree(matrixDegree.get());
                            }
                            targetMatrixSumScore = targetMatrixSumScore.add(targetMatrixScore);
                            reportExamStudentDto.setScore(reportExamStudentDto.getScore().add(targetMatrixScore));

                            if (Objects.nonNull(reportExamStudentFinalScoreDto)) {
                                reportExamStudentFinalScoreDto.setTargetMatrixSumScore(targetMatrixSumScore);
                                reportExamStudentFinalScoreAvgDto.setTargetMatrixSumScore(reportExamStudentFinalScoreAvgDto.getTargetMatrixSumScore().add(targetMatrixSumScore));
                            }
                            ReportExamStudentUsualScoreDto reportExamStudentUsualScoreAvgDto = reportExamStudentTargetDto.getUsualScore();

                            //平时作业总和+累加
                            List<ReportExamStudentUsualScoreObjDto> reportExamStudentUsualScoreObjDtoList = reportExamStudentUsualScoreDto.getScoreList();
                            if (Objects.nonNull(reportExamStudentUsualScoreAvgDto) && !CollectionUtils.isEmpty(reportExamStudentUsualScoreObjDtoList)) {
                                List<ReportExamStudentUsualScoreObjDto> reportExamStudentUsualScoreObjAvgDtoList = reportExamStudentUsualScoreAvgDto.getScoreList();
                                for (int i = 0; i < reportExamStudentUsualScoreObjDtoList.size(); i++) {
                                    ReportExamStudentUsualScoreObjDto reportExamStudentUsualScoreObjDto = reportExamStudentUsualScoreObjDtoList.get(i);
                                    reportExamStudentDto.setScore(reportExamStudentDto.getScore().add(reportExamStudentUsualScoreObjDto.getMatrixScore()));
                                    BigDecimal targetEvaluationValue = reportExamStudentUsualScoreObjDto.getScore().divide(reportExamStudentUsualScoreObjDto.getTargetScore(), 4, BigDecimal.ROUND_HALF_UP).multiply(reportExamStudentUsualScoreObjDto.getTargetWeight().divide(courseTargetWordDto.getTargetSumWeight(), 4, BigDecimal.ROUND_HALF_UP)).setScale(2, BigDecimal.ROUND_HALF_UP);
                                    matrixDegree.set(matrixDegree.get().add(targetEvaluationValue));
                                    reportExamStudentUsualScoreObjAvgDtoList.get(i).setMatrixScore(reportExamStudentUsualScoreObjAvgDtoList.get(i).getMatrixScore().add(reportExamStudentUsualScoreObjDto.getMatrixScore()));
                                }
                            }
                            reportExamStudentTargetDtoMap.put(k, reportExamStudentTargetDto);
                        }
                        targetList.add(new ReportExamStudentTargetDto(courseTargetWordDto, reportExamStudentFinalScoreDto, reportExamStudentUsualScoreDto, matrixDegree.get()));
                    }
                    reportExamStudentDto.setTargetList(targetList);
                });
                reportExamStudentDto.setScore(reportExamStudentDto.getScore().setScale(2, BigDecimal.ROUND_HALF_UP));
                Collections.sort(reportExamStudentDto.getTargetList());
                trExamStudentList.add(new TRExamStudent(trBasicInfo.getId(), reportExamStudentDto));
            }
            //加入目标分&平均分&各课程目标平均分
            BigDecimal size = new BigDecimal(examStudentList.size());
            List<ReportExamStudentTargetDto> targetDtoList = this.targetDtoParams(reportExamStudentTargetDtoMap, size);
            BigDecimal score = new BigDecimal(examStudentList.stream().mapToDouble(s -> s.getScore().doubleValue()).sum()).divide(size, 2, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP);
            ReportExamStudentDto reportExamStudentTargetDto = new ReportExamStudentDto("目标分", "目标分", null, SystemConstant.PERCENT, targetDtoList);
            ReportExamStudentDto reportExamStudentAvgDto = new ReportExamStudentDto("平均分", "平均分", null, score, targetDtoList);
            ReportExamStudentDto reportExamStudentTargetAvgDto = new ReportExamStudentDto("各课程目标平均分", "各课程目标平均分", null, null, targetDtoList);

            examStudentList.addAll(Arrays.asList(reportExamStudentTargetDto, reportExamStudentAvgDto, reportExamStudentTargetAvgDto));
            trExamStudentList.addAll(Arrays.asList(new TRExamStudent(trBasicInfo.getId(), reportExamStudentTargetDto), new TRExamStudent(trBasicInfo.getId(), reportExamStudentAvgDto), new TRExamStudent(trBasicInfo.getId(), reportExamStudentTargetAvgDto)));
            this.remove(new QueryWrapper<TRExamStudent>().lambda().eq(TRExamStudent::getrBasicInfoId, trBasicInfo.getId()));
            this.saveBatch(trExamStudentList);
        }
        return examStudentList;
    }

    /**
     * 根据报告id删除数据
     *
     * @param rBasicInfoId
     * @return
     */
    @Override
    @Transactional
    public Boolean remove(Long rBasicInfoId) {
        return trExamStudentService.remove(new QueryWrapper<TRExamStudent>().lambda().eq(TRExamStudent::getrBasicInfoId, rBasicInfoId));
    }

    @Override
    @Transactional
    public Boolean remove(Long obeCultureProgramId, Long courseId, String paperNumber) {
        TRBasicInfo trBasicInfo = trBasicInfoService.queryBasicInfo(obeCultureProgramId, courseId, paperNumber);
        return Objects.nonNull(trBasicInfo) ? trExamStudentService.remove(trBasicInfo.getId()) : false;
    }

    @Override
    public StudentRequirement getRequirementsStudentReport(Long cultureProgramId, String studentCode, Long semesterId) {
        List<CourseRequirementMatrixDto> courseRequirementMatrixDtoList = obeCourseRequirementMatrixService.getMatrix(cultureProgramId);

        List<StudentCourseInfoDto> studentCourseInfoDtoList = this.baseMapper.listStudentCourseByCultureProgramIdAndStudentCodeAndSemesterId(cultureProgramId, studentCode, semesterId);
        for (StudentCourseInfoDto studentCourseInfoDto : studentCourseInfoDtoList) {
            studentCourseInfoDto.setCourseTargetRequirementList(getRequirementsStudentReport(studentCourseInfoDto.getCourseId(), courseRequirementMatrixDtoList));
            String resultDetail = studentCourseInfoDto.getResultDetail();
            if (!StringUtils.isEmpty(resultDetail)) {
                List<ReportExamStudentTargetDto> reportExamStudentTargetDtoList = JSON.parseArray(resultDetail, ReportExamStudentTargetDto.class);
                // 指标点
                for (ReportExamStudentTargetDto reportExamStudentTargetDto : reportExamStudentTargetDtoList) {
                    ObeCourseTarget obeCourseTarget = obeCourseTargetService.getById(reportExamStudentTargetDto.getTargetId());
                    for (CourseTargetRequirement courseTargetRequirement : studentCourseInfoDto.getCourseTargetRequirementList()) {
                        if (courseTargetRequirement.getSubRequirementId().equals(obeCourseTarget.getObeCultureProgramRequirementId())) {
                            courseTargetRequirement.setCourseTargetDegree(reportExamStudentTargetDto.getMatrixDegree() != null && courseTargetRequirement.getScale() != null ? reportExamStudentTargetDto.getMatrixDegree().multiply(new BigDecimal(courseTargetRequirement.getScale())).setScale(2, RoundingMode.HALF_UP).doubleValue() : null);
                        } else {
                            courseTargetRequirement.setCourseTargetDegree(courseTargetRequirement.getCourseTargetDegree() != null ? courseTargetRequirement.getCourseTargetDegree() : null);
                        }
                    }
                }
            }
        }

        StudentRequirement studentRequirement = new StudentRequirement();
        // 各科目达成度
        List<StudentCourseRequirement> studentCourseRequirementList = new ArrayList<>();
        for (StudentCourseInfoDto studentCourseInfoDto : studentCourseInfoDtoList) {
            StudentCourseRequirement studentCourseRequirement = new StudentCourseRequirement();
            studentCourseRequirement.setCourseCode(studentCourseInfoDto.getCourseCode());
            studentCourseRequirement.setCourseName(studentCourseInfoDto.getCourseName());
            studentCourseRequirement.setSemesterName(studentCourseInfoDto.getSemesterName());
            studentCourseRequirement.setScore(studentCourseInfoDto.getScore());

            Map<String, List<CourseTargetRequirement>> map = studentCourseInfoDto.getCourseTargetRequirementList().stream().collect(Collectors.groupingBy(m -> m.getRequirementSortNum() + "##" + m.getRequirementName()));
            List<StudentCourseRequirementDetail> requirementDetailList = new ArrayList<>();
            for (Map.Entry<String, List<CourseTargetRequirement>> entry : map.entrySet()) {
                String[] s = entry.getKey().split("##");
                StudentCourseRequirementDetail studentCourseRequirementDetail = new StudentCourseRequirementDetail(Integer.valueOf(s[0]), s[1], null, null);
                studentCourseRequirementDetail.setSubRequirementDetailList(entry.getValue().stream().map(m -> new StudentCourseRequirementDetail(m.getSubRequirementSortNum(), m.getSubRequirementName(), m.getRequirementContent(), m.getCourseTargetDegree())).collect(Collectors.toList()));
                requirementDetailList.add(studentCourseRequirementDetail);
            }
            requirementDetailList.sort(Comparator.comparingInt(StudentCourseRequirementDetail::getRequirementSortNum));
            studentCourseRequirement.setRequirementDetailList(requirementDetailList);
            studentCourseRequirementList.add(studentCourseRequirement);
        }
        studentRequirement.setStudentCourseRequirementList(studentCourseRequirementList);

        // 汇总达成度
        List<CourseTargetRequirement> courseTargetRequirementListAll = new ArrayList<>();
        for (StudentCourseInfoDto studentCourseInfoDto : studentCourseInfoDtoList) {
            courseTargetRequirementListAll.addAll(studentCourseInfoDto.getCourseTargetRequirementList());
        }

        Map<String, BigDecimal> requirementDegreeMap = new HashMap<>();
        ObeCourseRequirementReport obeCourseRequirementReport = obeCourseRequirementReportService.findByCultureProgramId(cultureProgramId);
        if (obeCourseRequirementReport != null) {
            ObeCourseRequirementDto obeCourseRequirementDto = JSON.parseObject(obeCourseRequirementReport.getRequirementRadar(), ObeCourseRequirementDto.class);
            for (ObeRequirementDto obeRequirement : obeCourseRequirementDto.getObeRequirements()) {
                requirementDegreeMap.put(obeRequirement.getName(), obeRequirement.getMatrixDegree());
            }
        }

        Map<String, List<CourseTargetRequirement>> stringDoubleMap = courseTargetRequirementListAll.stream().collect(Collectors.groupingBy(m -> m.getRequirementSortNum() + "##" + m.getRequirementName()));
        List<StudentTotalRequirement> studentTotalRequirementList = new ArrayList<>();
        for (Map.Entry<String, List<CourseTargetRequirement>> entry : stringDoubleMap.entrySet()) {
            String[] keys = entry.getKey().split("##");
            StudentTotalRequirement studentTotalRequirement = new StudentTotalRequirement();
            studentTotalRequirement.setRequirementName(keys[1]);
            studentTotalRequirement.setRequirementSortNum(Integer.valueOf(keys[0]));
            if (requirementDegreeMap.containsKey(keys[1])) {
                studentTotalRequirement.setProfessionalDegree(requirementDegreeMap.get(keys[1]) != null ? requirementDegreeMap.get(keys[1]).doubleValue() : null);
            }
            studentTotalRequirementList.add(studentTotalRequirement);
        }
        studentTotalRequirementList.sort(Comparator.comparingInt(StudentTotalRequirement::getRequirementSortNum));
        studentRequirement.setStudentTotalRequirementList(studentTotalRequirementList);
        return studentRequirement;
    }

    private List<CourseTargetRequirement> getRequirementsStudentReport(Long courseId, List<CourseRequirementMatrixDto> courseRequirementMatrixDtoList) {
        List<CourseTargetRequirement> courseTargetRequirementList = new ArrayList<>();
        List<CourseRequirementMatrixDto> requirementMatrixDtoList = courseRequirementMatrixDtoList.stream().filter(m -> m.getCourseId().equals(courseId)).collect(Collectors.toList());
        for (CourseRequirementMatrixDto courseRequirementMatrixDto : requirementMatrixDtoList) {
            for (MatrixRequirementDto requirement : courseRequirementMatrixDto.getRequirements()) {
                for (MatrixSubRequirementDto subRequirement : requirement.getSubRequirements()) {
                    courseTargetRequirementList.add(new CourseTargetRequirement(subRequirement.getSubRequirementId(), subRequirement.getName(), subRequirement.getSubRequirementSortNum(), subRequirement.getContent(), requirement.getId(), requirement.getName(), requirement.getSortNum(), subRequirement.getScale().doubleValue() != 0 ? subRequirement.getScale().doubleValue() : null));
                }
            }
        }
        return courseTargetRequirementList;
    }

    @Override
    public IPage<StudentRequirementInfoDto> getRequirementsStudentPage(Long cultureProgramId, String studentCodeOrName, Integer pageNumber, Integer pageSize) {
        return this.baseMapper.requirementStudentPage(new Page<>(pageNumber, pageSize), cultureProgramId, studentCodeOrName);
    }

//    /**
//     * 知识点纬度数据构建
//     *
//     * @param map
//     * @param targetWordMap
//     * @param reportExamStudentDto
//     * @param reportExamStudentUsualScoreSumDto
//     * @param reportExamStudentTargetDtoMap
//     * @param finalScoreResult
//     */
//    protected void dimensionDataBuild(Map<Long, Map<Long, DimensionDto>> map, Map<Long, CourseTargetWordDto> targetWordMap,
//                                      ReportExamStudentDto reportExamStudentDto, ReportExamStudentUsualScoreDto reportExamStudentUsualScoreSumDto,
//                                      Map<Long, ReportExamStudentTargetDto> reportExamStudentTargetDtoMap, FinalScoreResult finalScoreResult) {
//        //课程目标循环
//        map.forEach((k, v) -> {
//            List<DimensionDto> dimensionDtoList = new ArrayList<>(v.values());
//            Collections.sort(dimensionDtoList);
//            List<ReportExamStudentTargetDto> targetList = CollectionUtils.isEmpty(reportExamStudentDto.getTargetList()) ? new ArrayList<>() : reportExamStudentDto.getTargetList();
//            if (targetWordMap.containsKey(k)) {
//                List<ReportExamStudentUsualScoreObjDto> scoreObjDtoList = reportExamStudentUsualScoreSumDto.getScoreMap().get(k);
//                CourseTargetWordDto courseTargetWordDto = targetWordMap.get(k);
//                ReportExamStudentFinalScoreDto reportExamStudentFinalScoreDto = new ReportExamStudentFinalScoreDto(courseTargetWordDto.getFinalScoreDto(), dimensionDtoList, finalScoreResult.getFinalScoreDimension().get(k));
//                ReportExamStudentUsualScoreDto reportExamStudentUsualScoreDto = new ReportExamStudentUsualScoreDto(scoreObjDtoList);
//
//                BigDecimal dimensionMatrixSumScore = new BigDecimal(0);
//                if (!reportExamStudentTargetDtoMap.containsKey(k)) {
//                    //各知识点转换分数
//                    for (DimensionDto dimensionDto : dimensionDtoList) {
//                        Double matrixScore = new BigDecimal(dimensionDto.getDimensionScore()).multiply(courseTargetWordDto.getFinalScoreDto().getFinalScoreSumWeight()).divide(SystemConstant.PERCENT, 2, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
//                        dimensionDto.setDimensionMatrixScore(matrixScore);
//                        dimensionMatrixSumScore = dimensionMatrixSumScore.add(new BigDecimal(matrixScore)).setScale(2, BigDecimal.ROUND_HALF_UP);
//                        reportExamStudentDto.setScore(reportExamStudentDto.getScore() + matrixScore);
//                    }
//                    //平时作业总和+累加
//                    Double usualMatrixSumScore = scoreObjDtoList.stream().mapToDouble(ReportExamStudentUsualScoreObjDto::getMatrixScore).sum();
//                    reportExamStudentDto.setScore(reportExamStudentDto.getScore() + usualMatrixSumScore);
//                    reportExamStudentFinalScoreDto.setTargetDimensionMatrixSumScore(dimensionMatrixSumScore.doubleValue());
//                    ReportExamStudentFinalScoreDto reportExamStudentFinalScoreTempDto = new ReportExamStudentFinalScoreDto(courseTargetWordDto.getFinalScoreDto(), dimensionDtoList.stream().map(DimensionDto::new).collect(Collectors.toList()), dimensionMatrixSumScore.doubleValue(), finalScoreResult.getFinalScoreDimension().get(k));
//                    ReportExamStudentTargetDto reportExamStudentTargetDto = new ReportExamStudentTargetDto(courseTargetWordDto, reportExamStudentFinalScoreTempDto, new ReportExamStudentUsualScoreDto(scoreObjDtoList.stream().map(ReportExamStudentUsualScoreObjDto::new).collect(Collectors.toList())));
//                    reportExamStudentTargetDtoMap.put(k, reportExamStudentTargetDto);
//                } else {
//                    ReportExamStudentTargetDto reportExamStudentTargetDto = reportExamStudentTargetDtoMap.get(k);
//                    ReportExamStudentFinalScoreDto reportExamStudentFinalScoreAvgDto = reportExamStudentTargetDto.getFinalScore();
//                    List<DimensionDto> dimensionDtos = reportExamStudentFinalScoreAvgDto.getDimensionList();
//                    //各知识点转换分数
//                    for (int i = 0; i < dimensionDtoList.size(); i++) {
//                        DimensionDto dimensionDto = dimensionDtoList.get(i);
//                        Double matrixScore = new BigDecimal(dimensionDto.getDimensionScore()).multiply(courseTargetWordDto.getFinalScoreDto().getFinalScoreSumWeight()).divide(SystemConstant.PERCENT, 2, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
//                        dimensionDto.setDimensionMatrixScore(matrixScore);
//                        reportExamStudentDto.setScore(reportExamStudentDto.getScore() + matrixScore);
//                        dimensionDtos.get(i).setDimensionMatrixScore(dimensionDtos.get(i).getDimensionMatrixScore() + matrixScore);
//                        dimensionMatrixSumScore = dimensionMatrixSumScore.add(new BigDecimal(matrixScore)).setScale(2, BigDecimal.ROUND_HALF_UP);
//                    }
//                    reportExamStudentFinalScoreDto.setTargetDimensionMatrixSumScore(dimensionMatrixSumScore.doubleValue());
//                    reportExamStudentFinalScoreAvgDto.setTargetDimensionMatrixSumScore(new BigDecimal(reportExamStudentFinalScoreAvgDto.getTargetDimensionMatrixSumScore()).add(dimensionMatrixSumScore).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
//                    ReportExamStudentUsualScoreDto reportExamStudentUsualScoreAvgDto = reportExamStudentTargetDto.getUsualScore();
//
//                    //平时作业总和+累加
//                    List<ReportExamStudentUsualScoreObjDto> reportExamStudentUsualScoreObjDtoList = reportExamStudentUsualScoreDto.getScoreList();
//                    List<ReportExamStudentUsualScoreObjDto> reportExamStudentUsualScoreObjAvgDtoList = reportExamStudentUsualScoreAvgDto.getScoreList();
//                    for (int i = 0; i < reportExamStudentUsualScoreObjDtoList.size(); i++) {
//                        ReportExamStudentUsualScoreObjDto reportExamStudentUsualScoreObjDto = reportExamStudentUsualScoreObjDtoList.get(i);
//                        reportExamStudentDto.setScore(reportExamStudentDto.getScore() + reportExamStudentUsualScoreObjDto.getMatrixScore());
//                        reportExamStudentUsualScoreObjAvgDtoList.get(i).setMatrixScore(new BigDecimal(reportExamStudentUsualScoreObjAvgDtoList.get(i).getMatrixScore()).add(new BigDecimal(reportExamStudentUsualScoreObjDto.getMatrixScore())).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
//                    }
//                    reportExamStudentTargetDtoMap.put(k, reportExamStudentTargetDto);
//                }
//                targetList.add(new ReportExamStudentTargetDto(courseTargetWordDto, reportExamStudentFinalScoreDto, reportExamStudentUsualScoreDto));
//            }
//            reportExamStudentDto.setTargetList(targetList);
//        });
//    }

//    /**
//     * 处理目标分&平均分&各课程目标平均分参数-知识点纬度
//     *
//     * @param reportExamStudentTargetDtoMap
//     * @param size
//     * @return
//     */
//    protected List<ReportExamStudentTargetDto> targetDimensionDtoParams(Map<Long, ReportExamStudentTargetDto> reportExamStudentTargetDtoMap, BigDecimal size) {
//        List<ReportExamStudentTargetDto> targetDtoList = new ArrayList<>();
//        reportExamStudentTargetDtoMap.forEach((k, v) -> {
//            ReportExamStudentFinalScoreDto reportExamStudentFinalScoreDto = v.getFinalScore();//课程目标考核要素的平均分
//            BigDecimal matrixAvgScore = new BigDecimal(reportExamStudentFinalScoreDto.getTargetDimensionMatrixSumScore()).divide(size, 2, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP);
////            //如果换算出来的知识点总分超过了期末考试目标分，则
////            if (reportExamStudentFinalScoreDto.getTargetScore().compareTo(matrixAvgScore) == -1) {
////                throw ExceptionResultEnum.ERROR.exception("[" + v.getTargetName() + "]知识点总平均分超过了期末考试目标分，请重新设置知识点");
////            }
//            reportExamStudentFinalScoreDto.setMatrixAvgScore(matrixAvgScore.doubleValue());
//
//            List<DimensionDto> dimensionDtoList = reportExamStudentFinalScoreDto.getDimensionList();
//            for (DimensionDto d : dimensionDtoList) {
//                d.setDimensionMatrixAvgScore(new BigDecimal(d.getDimensionMatrixScore()).divide(size, 2, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
//            }
//
//            v.setTargetMatrixAvgScore(reportExamStudentFinalScoreDto.getMatrixAvgScore());
//            ReportExamStudentUsualScoreDto reportExamStudentUsualScoreDto = v.getUsualScore();
//            List<ReportExamStudentUsualScoreObjDto> reportExamStudentUsualScoreObjDtoList = reportExamStudentUsualScoreDto.getScoreList();
//            for (ReportExamStudentUsualScoreObjDto r : reportExamStudentUsualScoreObjDtoList) {
//                r.setMatrixAvgScore(new BigDecimal(r.getMatrixScore()).divide(size, 2, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
//                v.setTargetMatrixAvgScore(v.getTargetMatrixAvgScore() + r.getMatrixAvgScore());
//            }
//            v.setTargetMatrixAvgScore(new BigDecimal(v.getTargetMatrixAvgScore()).setScale(1, BigDecimal.ROUND_DOWN).doubleValue());
//            targetDtoList.add(v);
//        });
//        Collections.sort(targetDtoList);
//        return targetDtoList;
//    }

    /**
     * 处理目标分&平均分&各课程目标平均分参数-课程目标纬度
     *
     * @param reportExamStudentTargetDtoMap
     * @param size
     * @return
     */
    protected List<ReportExamStudentTargetDto> targetDtoParams(Map<Long, ReportExamStudentTargetDto> reportExamStudentTargetDtoMap, BigDecimal size) {
        List<ReportExamStudentTargetDto> targetDtoList = new ArrayList<>();
        reportExamStudentTargetDtoMap.forEach((k, v) -> {
            ReportExamStudentFinalScoreDto reportExamStudentFinalScoreDto = v.getFinalScore();//课程目标考核要素的平均分
            if (Objects.nonNull(reportExamStudentFinalScoreDto)) {
                BigDecimal matrixAvgScore = reportExamStudentFinalScoreDto.getTargetMatrixSumScore().divide(size, 4, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP);
                reportExamStudentFinalScoreDto.setMatrixAvgScore(matrixAvgScore);
                v.setTargetMatrixAvgScore(reportExamStudentFinalScoreDto.getMatrixAvgScore());
            }
            ReportExamStudentUsualScoreDto reportExamStudentUsualScoreDto = v.getUsualScore();
            if (Objects.nonNull(reportExamStudentUsualScoreDto) && !CollectionUtils.isEmpty(reportExamStudentUsualScoreDto.getScoreList())) {
                List<ReportExamStudentUsualScoreObjDto> reportExamStudentUsualScoreObjDtoList = reportExamStudentUsualScoreDto.getScoreList();
                for (ReportExamStudentUsualScoreObjDto r : reportExamStudentUsualScoreObjDtoList) {
                    r.setMatrixAvgScore(r.getMatrixScore().divide(size, 4, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP));
                    v.setTargetMatrixAvgScore(Objects.nonNull(v.getTargetMatrixAvgScore()) ? v.getTargetMatrixAvgScore() : new BigDecimal(0).add(r.getMatrixAvgScore()));
                }
            }
            v.setTargetMatrixAvgScore(v.getTargetMatrixAvgScore().setScale(2, BigDecimal.ROUND_HALF_UP));
            targetDtoList.add(v);
        });
        Collections.sort(targetDtoList);
        return targetDtoList;
    }
}
